@@ -1,4 +1,4 @@
-//Axes.cpp, Copyright 2000-2007 R.Lackner
+//Axes.cpp, Copyright 2000-2008 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -88,7 +88,7 @@ GridLine::DoPlot(anyOutput *o)
 		ncpts = 0;
 		cpts = MakeArc(ix, iy, ir, 0x0f, &ncpts);
 		SetMinMaxRect(&rDims, ix-ir, iy-ir, ix+ir, iy+ir);
-		IncrementMinMaxRect(&rDims, 3);
+		IncrementMinMaxRect(&rDims, 6 + o->un2ix(LineDef.width));
 		o->oPolyline(cpts, (int)ncpts);
 		return;
 		}
@@ -133,7 +133,7 @@ GridLine::DoPlot(anyOutput *o)
 			o->oPolyline(pts+2, 2);
 			SetMinMaxRect(&rDims, pts[2].x, pts[2].y, pts[3].x, pts[3].y);
 			}
-		IncrementMinMaxRect(&rDims, 3);
+		IncrementMinMaxRect(&rDims, 6 + o->un2ix(LineDef.width));
 		}
 }
 
@@ -360,7 +360,7 @@ GridLine3D:: DoPlot(anyOutput *o)
 			UpdateMinMaxRect(&rDims, gl3[3].x, gl3[3].y);
 			}
 		}
-	IncrementMinMaxRect(&rDims, 4);
+	IncrementMinMaxRect(&rDims, 6 + o->un2ix(LineDef.width));
 }
 
 bool
@@ -420,7 +420,7 @@ GridRadial::DoPlot(anyOutput *o)
 		pts[1].x = o->co2ix(axdef->Center.fx + parent->GetSize(SIZE_GRECT_LEFT));
 		pts[1].y = o->co2iy(axdef->Center.fy + parent->GetSize(SIZE_GRECT_TOP));
 		SetMinMaxRect(&rDims, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
-		IncrementMinMaxRect(&rDims, 3);
+		IncrementMinMaxRect(&rDims, 6 + o->un2ix(LineDef.width));
 		o->oPolyline(pts, 2);
 		}
 }
@@ -484,9 +484,16 @@ Tick::Tick(int src):GraphObj(0L, 0L)
 
 Tick::~Tick()
 {
+	int i;
+
+	if(Polygons) {
+		for(i = 0; i < numPG; i++) if(Polygons[i]) DeleteGO(Polygons[i]);
+		free(Polygons);			Polygons = 0L;	numPG = 0;
+		}
 	Command(CMD_FLUSH, 0L, 0L);
-	if(mo) DelBitmapClass(mo);	mo = 0L;
+	if(mo) DelBitmapClass(mo);		mo = 0L;
 	if(bModified) Undo.InvalidGO(this);
+	if(seg) free(seg);			seg = 0L;
 }
 	
 double
@@ -545,6 +552,7 @@ Tick::SetColor(int select, DWORD col)
 void
 Tick::DoMark(anyOutput *o, bool mark)
 {
+	if(!bValidTick) return;
 	if(mark){
 		if(mo) DelBitmapClass(mo);					mo = 0L;
 		memcpy(&mrc, &rDims, sizeof(RECT));
@@ -562,13 +570,22 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 	TextDEF *LabelDef;
 	GraphObj **tmpPlots;
 	AxisDEF *axis;
+	DWORD pg_color;
+	int i;
 
 	switch(cmd){
 	case CMD_SCALE:
 		if(label) label->Command(cmd, tmpl, o);
 		if(Grid) Grid->Command(cmd, tmpl, o);
 		size *= ((scaleINFO*)tmpl)->sy.fy;
+		if(Polygons) for(i = 0; i < numPG; i++) if(Polygons[i]) (Polygons[i])->Command(cmd, tmpl, o);;
 		break;
+	case CMD_ADDTOLINE:
+		return StoreSeg((lfPOINT*)tmpl);
+	case CMD_RECALC:
+		if(Polygons) Undo.DropListGO(this, (GraphObj***)&Polygons, &numPG, UNDO_CONTINUE);
+		n_seg = 0;		bModified = true;
+		return true;
 	case CMD_SET_AXDEF:
 		if(axis = (AxisDEF*)tmpl) {
 			flags = (flags & AXIS_MINORTICK) | axis->flags;
@@ -584,11 +601,24 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		flags &= ~0x07;
 		flags |= (0x07 & *((DWORD*)tmpl));
 		return true;
+	case CMD_UPDPG:
+		if(parent && parent->Id == GO_AXIS) pg_color = ((Axis*)parent)->GradColor(value);
+		else pg_color = 0x00ffffff;
+		if(Polygons) for(i = 0; i < numPG; i++) if(Polygons[i]) {
+			Polygons[i]->SetColor(COL_POLYGON, pg_color);
+			}
+		return true;
+	case CMD_DRAWPG:
+		if(Polygons) for(i = 0; i < numPG; i++) if(Polygons[i]) {
+			Polygons[i]->DoPlot(o);
+			}
+		// we lost the line definition from the parent axis
+		if(parent) parent->Command(CMD_RESET_LINE, 0L, o);
+		return true;
 	case CMD_TICK_TYPE:
 		if(tmpl) type = *((int*)tmpl);
 		return true;
-	case CMD_SETSCROLL:
-	case CMD_REDRAW:
+	case CMD_SETSCROLL:		case CMD_REDRAW:
 		//this commands are usually issued from a child or from Undo
 		bModified = true;
 		return parent ? parent->Command(CMD_REDRAW, 0L, o) : false;
@@ -613,10 +643,11 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 				label = 0L;
 				return parent->Command(CMD_REDRAW, 0L, o);
 				}
+			if(DeleteGOL((GraphObj***) &Polygons, numPG, (GraphObj *)tmpl, o)) 
+				return parent->Command(CMD_REDRAW, 0L, o);
 			}
 		return false;
-	case CMD_SET_GO3D:
-	case CMD_GET_GRIDLINE:
+	case CMD_SET_GO3D:		case CMD_GET_GRIDLINE:
 		if(parent) return parent->Command(cmd, tmpl, o);
 		return false;
 	case CMD_SET_GRIDTYPE:
@@ -628,6 +659,7 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		Id = GO_TICK;
 		if(Grid) Grid->Command(cmd, tmpl, o);
 		if(label) label->Command(cmd, tmpl, o);
+		if(Polygons) for(i = 0; i < numPG; i++) if(Polygons[i]) (Polygons[i])->Command(cmd, tmpl, o);;
 		return true;
 	case CMD_TEXTTHERE:
 		if(label && label->Command(cmd, tmpl, o)) return true;
@@ -643,12 +675,17 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 				o->ShowMark(this, MRK_GODRAW);
 				return true;
 				}
+			if(Polygons && !CurrGO) for(i = numPG-1; i >= 0; i--)
+				if(Polygons[i]) if(Polygons[i]->Command(cmd, tmpl, o))return true;
 			break;
 			}
 		return false;
 	case CMD_TLB_TXTDEF:
 		if(label) return label->Command(CMD_SETTEXTDEF, tmpl, o);
-		else return false;
+		return false;
+	case CMD_GETTEXT:
+		if(label) return label->Command(cmd, tmpl, o);
+		return false;
 	case CMD_SETTEXT:
 		if(label) return label->Command(cmd, tmpl, o);
 		if(!(LabelDef = (TextDEF *)calloc(1, sizeof(TextDEF))))return false;
@@ -676,6 +713,23 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 	return false;
 }
 
+void
+Tick::Track(POINT *p, anyOutput *o)
+{
+	POINT tpts[2];
+	int iw;
+
+	if(!bValidTick || !parent || !o) return;
+	iw = 6 + o->un2ix(o->LineWidth);
+	defs.UpdRect(o, rDims.left, rDims.top, rDims.right, rDims.bottom);
+	tpts[0].x = pts[0].x+p->x;	tpts[0].y = pts[0].y+p->y;
+	tpts[1].x = pts[1].x+p->x;	tpts[1].y = pts[1].y+p->y;
+	defs.UpdRect(o, tpts[0].x-iw, tpts[0].y-iw, tpts[1].x-iw, tpts[1].y-iw);
+	defs.UpdRect(o, tpts[0].x+iw, tpts[0].y+iw, tpts[1].x+iw, tpts[1].y+iw);
+	o->ShowLine(tpts, 2, parent->GetColor(COL_AXIS));
+	if((flags & AXIS_MINORTICK) == 0 && label) label->Track(p, o);
+}
+
 void
 Tick::DoPlot(double six, double csx, anyOutput *o)
 {
@@ -686,7 +740,7 @@ Tick::DoPlot(double six, double csx, anyOutput *o)
 	if(mo) DelBitmapClass(mo);		mo = 0L;
 	if(ls) delete(ls);				ls = 0L;
 	ip2.x = ip2.y = ip2.z = 0;
-	if(!((Axis*)parent)->GetValuePos(value, &fix, &fiy, &fiz, o))return;
+	if(!(bValidTick = ((Axis*)parent)->GetValuePos(value, &fix, &fiy, &fiz, o))) return;
 	lbx = fix;		lby = fiy;
 	if(flags & AXIS_ANGULAR) {
 		dp1.fx = o->co2fix(parent->GetSize(SIZE_XCENT)+parent->GetSize(SIZE_GRECT_LEFT));
@@ -742,34 +796,146 @@ Tick::DoPlot(double six, double csx, anyOutput *o)
 	pts[1].x += ip2.x;			pts[1].y += ip2.y;			p32.z += ip2.z;
 	p32.x += ip2.x;				p32.y += ip2.y;
 	SetMinMaxRect(&rDims, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
-	IncrementMinMaxRect(&rDims, 6);
+	IncrementMinMaxRect(&rDims, 6 + o->un2ix(o->LineWidth));
 	if(parent && parent->Id == GO_AXIS && (flags & AXIS_3D)){
 		if(ls = new line_segment(this, data, &((Axis*)parent)->axline, &p31, &p32)){
 			ls->DoPlot(o);
 			}
 		}
 	else o->oSolidLine(pts);
-	if(flags & AXIS_MINORTICK) return;
-	if(flags & AXIS_GRIDLINE) {
-		if(!Grid){
-			if(flags & AXIS_3D) {
-				Grid = new GridLine3D(this, data, gl_type, flags);
+	if(!(flags & AXIS_MINORTICK)){
+		if(flags & AXIS_GRIDLINE) {
+			if(!Grid){
+				if(flags & AXIS_3D) {
+					Grid = new GridLine3D(this, data, gl_type, flags);
+					}
+				else if((flags & AXIS_ANGULAR) == AXIS_ANGULAR){
+					Grid = new GridRadial(this, data, gl_type, flags);
+					}
+				else {
+					Grid = new GridLine(this, data, gl_type, flags);
+					}
 				}
-			else if((flags & AXIS_ANGULAR) == AXIS_ANGULAR){
-				Grid = new GridRadial(this, data, gl_type, flags);
+			if(Grid) Grid->DoPlot(o);
+			// we lost the line definition from the parent axis
+			if(parent) parent->Command(CMD_RESET_LINE, 0L, o);
+			}
+		if(label){
+			if(flags & AXIS_3D) label->SetSize(SIZE_ZPOS, fiz); 
+			label->DoPlot(o);
+			}
+		}
+}
+
+//return true if two points are close together
+#define X_TOL 1.0E-12
+#define Y_TOL 1.0E-12
+bool
+Tick::CmpPoints(double x1, double y1, double x2, double y2)
+{
+	if(fabs(x2 - x1) > X_TOL) return false;
+	if(fabs(y2 - y1) > Y_TOL) return false;
+	return true;
+}
+#undef X_TOL
+#undef Y_TOL
+
+bool
+Tick::StoreSeg(lfPOINT *line)
+{
+	int i;
+
+	if(n_seg >= s_seg) {
+		if(!(seg = (double*)realloc(seg, ((s_seg += 50)<<2)*sizeof(double))))return false;
+		}
+	i = n_seg << 2;
+	seg[i++] = line[0].fx;		seg[i++] = line[0].fy;
+	seg[i++] = line[1].fx;		seg[i++] = line[1].fy;
+	n_seg++;			return false;
+}
+
+//collect line segments to polygons
+bool
+Tick::ProcSeg()
+{
+	lfPOINT *pg;
+	int i, j, l, n, n0, n1, n_pg, level;
+	bool cont1, cont2;
+	FillDEF pg_fill = {0, 0x0, 1.0, 0L, 0x0L};
+	LineDEF pg_line = {defs.GetSize(SIZE_HAIRLINE), 1.0, 0x0L, 0x0L};
+
+	if(n_seg < 3) {
+		n_seg = 0;	return false;
+		}
+	if(!(pg = (lfPOINT*)malloc((n_seg+2)*sizeof(lfPOINT)))) return false;
+	if(!parent || parent->Id != GO_AXIS) return false;
+	pg_fill.color = pg_fill.color2 = ((Axis*)parent)->GradColor(value);
+	pg_line.color = ((pg_fill.color & 0x00fefefeL)>>1);
+	while(n_seg > 2) {
+		pg[0].fx = seg[0];	pg[0].fy = seg[1];
+		pg[1].fx = seg[2];	pg[1].fy = seg[3];
+		n0 = level = 0; n = 1;	n_pg = 2;
+		do {
+			do {
+				cont2 = cont1 = false;		n1 = (n<<2);
+				if(CmpPoints(pg[n_pg-1].fx, pg[n_pg-1].fy, seg[n1], seg[n1+1])) {
+					pg[n_pg].fx = seg[n1+2];	pg[n_pg].fy = seg[n1+3];
+					n++;	n_pg++;		cont2=true;
+					}
+				else if(CmpPoints(pg[n_pg-1].fx, pg[n_pg-1].fy, seg[n1+2], seg[n1+3] )) {
+					pg[n_pg].fx = seg[n1];		pg[n_pg].fy = seg[n1+1];
+					n++;	n_pg++;		cont2=true;
+					}
+				else if(CmpPoints(pg[0].fx, pg[0].fy, seg[n1], seg[n1+1])) {
+					for(i = n_pg; i > 0; i--) {
+						pg[i].fx = pg[i-1].fx;	pg[i].fy = pg[i-1].fy;
+						}
+					pg[0].fx = seg[n1+2];		pg[0].fy = seg[n1+3];
+					n++;	n_pg++;		cont2=true;
+					}
+				else if(CmpPoints(pg[0].fx, pg[0].fy, seg[n1+2], seg[n1+3])) {
+					for(i = n_pg; i > 0; i--) {
+						pg[i].fx = pg[i-1].fx;	pg[i].fy = pg[i-1].fy;
+						}
+					pg[0].fx = seg[n1];			pg[0].fy = seg[n1+1];
+					n++;	n_pg++;		cont2=true;
+					}
+				if(cont2) level = 0;
+				}while(cont2 && n_pg < s_seg && n < n_seg);
+			if(n > n0 && n < n_seg) {
+				for(i = n0<<2, j = n<<2, l = (n_seg<<2); j < l; i++, j++) {
+					seg[i] = seg[j];
+					}
+				n_seg -= (n-n0);	n0 = n = 0;
+				}
+			else if(n == n_seg){
+				n_seg = n0;			n0 = n = 0;
 				}
 			else {
-				Grid = new GridLine(this, data, gl_type, flags);
+				n0++;				n++;
+				}
+			if(CmpPoints(pg[0].fx, pg[0].fy, pg[n_pg-1].fx, pg[n_pg-1].fy)) cont1 = false;
+			else if(n < n_seg && n_seg) cont1 = true;
+			else {
+				if(n_seg && level < 3) {
+					level++;	n = 0;	cont1 = true;
+					}
+				else cont1 = false;
+				}
+			}while(cont1);
+		if(n_pg > 2) {
+			if(CmpPoints(pg[0].fx, pg[0].fy, pg[n_pg-1].fx, pg[n_pg-1].fy)) n_pg--;
+			if(n_pg > 2) {
+				Polygons = (DataPolygon**)realloc(Polygons, (numPG+2)*sizeof(DataPolygon*));
+				if(Polygons[numPG] = new DataPolygon(this, data, (lfPOINT*)memdup(pg, n_pg*sizeof(lfPOINT), 0), n_pg, 0L)){
+					Polygons[numPG]->type = 12;
+					Polygons[numPG]->Command(CMD_SET_LINE, &pg_line, 0L);
+					Polygons[numPG++]->Command(CMD_PG_FILL, &pg_fill, 0L);
+					}
 				}
 			}
-		if(Grid) Grid->DoPlot(o);
-		// we lost the line definition from the parent axis
-		if(parent) parent->Command(CMD_RESET_LINE, 0L, o);
-		}
-	if(label){
-		if(flags & AXIS_3D) label->SetSize(SIZE_ZPOS, fiz); 
-		label->DoPlot(o);
 		}
+	free(pg);	n_seg = 0;	return true;
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1031,7 +1197,7 @@ Axis::DoPlot(anyOutput *o)
 			if(Ticks[i]) Ticks[i]->DoPlot(0.0, 1.0, o);
 		rDims.left = ix-i;	rDims.right = ix +i;
 		rDims.top = iy -i;	rDims.bottom = iy+i;
-		IncrementMinMaxRect(&rDims, o->un2ix(axline.width*4.0));
+		IncrementMinMaxRect(&rDims, 6 + o->un2ix(sizAxLine));
 		return;
 		}
 	if(axis->flags & AXIS_3D){
@@ -1141,7 +1307,7 @@ Axis::DoPlot(anyOutput *o)
 	pts3D[0].y = pts[0].y;				pts3D[1].y = pts[1].y;
 	pts3D[0].z = iround(flim[0].fz);	pts3D[1].z = iround(flim[1].fz);
 	SetMinMaxRect(&rDims, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
-	IncrementMinMaxRect(&rDims, 3);
+	IncrementMinMaxRect(&rDims, 6 + o->un2ix(sizAxLine));
 	//calculate sine and cosine for ticks in any direction of axis
 	si = flim[1].fy - flim[0].fy;
 	tmp = (flim[1].fx - flim[0].fx);
@@ -1149,8 +1315,9 @@ Axis::DoPlot(anyOutput *o)
 	csi = flim[1].fx - flim[0].fx;
 	tmp = (flim[1].fy - flim[0].fy);
 	csi = fabs(csi) > 0.0001 ? csi/sqrt(csi*csi + tmp*tmp) : 0.0;
-	//draw axis line
-	if(axis->breaks && axis->nBreaks)DrawBreaks(o);
+	//draw gradient bar, breaks and axis line
+	if(type == 4) GradientBar(o);
+	else if(axis->breaks && axis->nBreaks)DrawBreaks(o);
 	else if((axis->flags) & AXIS_3D){
 		if(!(l_segs = (line_segment**)calloc(1, sizeof(line_segment*))))return;
 		l_segs[0] = new line_segment(this, data, &axline, &pts3D[0], &pts3D[1]);
@@ -1164,7 +1331,8 @@ Axis::DoPlot(anyOutput *o)
 		axisLabel->DoPlot(o);
 		}
 	if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++) 
-		if(Ticks[i]) Ticks[i]->DoPlot(si, csi, o);
+		if(Ticks[i] && Ticks[i]->GetSize(SIZE_MINE) >= (axis->min - 1.0e-16)
+		&& Ticks[i]->GetSize(SIZE_MINE) <= (axis->max + 1.0e-16)) Ticks[i]->DoPlot(si, csi, o);
 }
 
 bool
@@ -1221,7 +1389,19 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		return true;
 	case CMD_SET_GRIDLINE:
 		if(tmpl) memcpy(&GridLine, tmpl, sizeof(LineDEF));
-		if(Ticks) for(i = 0; i< NumTicks; i++) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
+		return true;
+	case CMD_MRK_DIRTY:
+		if((type & 0xf) == 4 && !o) cmd = CMD_UPDPG;
+		else return false;
+	case CMD_DRAWPG:	case CMD_UPDPG:
+		if((type & 0xf) == 4 && Ticks) {
+			if(axis->flags & AXIS_INVERT){
+				for(i = NumTicks-1; i >= 0; i--) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
+				}
+			else {
+				for(i = 0; i< NumTicks; i++) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
+				}
+			}
 		return true;
 	case CMD_GET_GRIDLINE:
 		if(tmpl) memcpy(tmpl, &GridLine, sizeof(LineDEF));
@@ -1230,24 +1410,14 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		mev = (MouseEvent *) tmpl;
 		switch (mev->Action) {
 		case MOUSE_LBUP:
-			if(axis->flags & AXIS_ANGULAR) {
-				i = (mev->x - pts[0].x) * (mev->x - pts[0].x) + 
-					(mev->y - pts[0].y) * (mev->y - pts[0].y);
-				i = isqr(i) - pts[1].x;
-				if(i < 4 && i > -4){
-					o->ShowMark(this, MRK_GODRAW);
-					return true;
-					}
-				}
-			else if(IsInRect(&rDims, mev->x, mev->y) && !CurrGO && 
-				IsCloseToLine(&pts[0], &pts[1], mev->x, mev->y)) {
+			if(!CurrGO && ObjThere(mev->x, mev->y)) {
 				o->ShowMark(this, MRK_GODRAW);
 				return true;
 				}
 			break;
 			}
 		if(axisLabel && axisLabel->Command(cmd, tmpl, o)) return true;
-		if(Ticks) for(i = 0; i < NumTicks; i++)
+		if(Ticks) for(i = (NumTicks-1); i >= 0; i--)
 			if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o)) return true;
 		return false;
 	case CMD_TEXTTHERE:
@@ -1315,6 +1485,18 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		if(axisLabel && axisLabel == tmpPlots[0])
 			return bModified = ReplaceGO((GraphObj**)&axisLabel, tmpPlots);
 		return false;
+	case CMD_MOVE:
+		if(moveable) {
+			bModified = true;
+			Undo.MoveObj(this, (lfPOINT*)tmpl, 0L);
+			}
+	case CMD_UNDO_MOVE:
+		if(moveable) {
+			axis->loc[0].fx += ((lfPOINT*)tmpl)[0].fx;	axis->loc[0].fy +=  ((lfPOINT*)tmpl)[0].fy;
+			axis->loc[1].fx += ((lfPOINT*)tmpl)[0].fx;	axis->loc[1].fy +=  ((lfPOINT*)tmpl)[0].fy;
+			}
+		CurrGO = this;
+		return parent->Command(CMD_REDRAW, 0, o);
 	case CMD_UPDATE:
 		UpdateTicks();
 	case CMD_SETSCROLL:
@@ -1338,6 +1520,12 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		if(scaleOut) delete(scaleOut);
 		scaleOut = drawOut = 0L;
 		return true;
+	case CMD_RECALC:
+		if(Ticks) {
+			for(i = 0; i < NumTicks; i++) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
+			return true;
+			}
+		break;
 	case CMD_AUTOSCALE:		//we receive this command to update ticks after rescaling
 		if(axis && (AxisDEF*)tmpl == axis && (axis->flags & AXIS_AUTOSCALE) && 
 			(axis->flags & AXIS_AUTOTICK)) return Command(CMD_FLUSH, tmpl, o);
@@ -1359,6 +1547,129 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 	return false;
 }
 
+void * 
+Axis::ObjThere(int x, int y)
+{
+	int i;
+	POINT p1 = {x, y};
+
+	if(axis->flags & AXIS_ANGULAR) {
+		i = (x - pts[0].x) * (x - pts[0].x) + (y - pts[0].y) * (y - pts[0].y);
+		i = isqr(i) - pts[1].x;
+		if(i < 4 && i > -4) return this;
+		}
+	else if((type & 0x04) == 4 && IsInPolygon(&p1, gradient_box, 5)) return this;
+	else if(IsInRect(&rDims, x, y) && IsCloseToLine(&pts[0], &pts[1], x, y)) {
+		return this;
+		}
+	return 0L;
+}
+
+void
+Axis::Track(POINT *p, anyOutput *o)
+{
+	POINT tpts[5];
+	int i, iw;
+
+	if(!parent || !o) return;
+	iw = 6 + o->un2ix(sizAxLine);
+	defs.UpdRect(o, rDims.left, rDims.top, rDims.right, rDims.bottom);
+	tpts[0].x = pts[0].x+p->x;	tpts[0].y = pts[0].y+p->y;
+	tpts[1].x = pts[1].x+p->x;	tpts[1].y = pts[1].y+p->y;
+	defs.UpdRect(o, tpts[0].x-iw, tpts[0].y-iw, tpts[1].x-iw, tpts[1].y-iw);
+	defs.UpdRect(o, tpts[0].x+iw, tpts[0].y+iw, tpts[1].x+iw, tpts[1].y+iw);
+	if(type & 0x04) {
+		for(i = 0; i < 5; i++) {
+			tpts[i].x = gradient_box[i].x + p->x;
+			tpts[i].y = gradient_box[i].y + p->y;
+			defs.UpdRect(o, tpts[i].x-iw, tpts[i].y-iw, tpts[i].x+iw, tpts[i].y+iw);
+			}
+		o->ShowLine(tpts, 5, colAxis);
+		}
+	else o->ShowLine(tpts, 2, colAxis);
+	if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++) {
+		if(Ticks[i] && Ticks[i]->GetSize(SIZE_MINE) >= (axis->min - 1.0e-16)
+			&& Ticks[i]->GetSize(SIZE_MINE) <= (axis->max + 1.0e-16)) Ticks[i]->Track(p, o);
+		}
+}
+
+DWORD
+Axis::GradColor(double val)
+{
+	DWORD retcol = 0x00cbcbcbL;
+	double y, f;
+
+	y = (val-axis->min)/(axis->max-axis->min);
+	if(axis->flags & AXIS_INVERT) y = 1.0-y;
+	if(grad_type & 0x10) y = 1.0-y;
+	switch(grad_type & 0x0f) {
+	case 0:
+		retcol = gCol_0;		break;
+	case 1:		case 4:
+		retcol = 0x00000000L;
+		if(y >= 1.0) retcol = 0x000000ffL;
+		else if(y >= 0.75) {
+			f = (y - 0.75)*1024.0;		retcol |= 0x000000ffL;
+			retcol |= (((f < 255.0)?((int)(255.0-f)):0)<<8);
+			}
+		else if(y >= 0.5) {
+			f = (y - 0.5)*1024.0;		retcol |= 0x0000ff00L;
+			retcol |= ((f < 255.0)?((int)f):0);
+			}
+		else if(y >= 0.0 && (grad_type &0x0f) == 4) {
+			f = y*512.0;
+			retcol |= (((f < 255.0)?((int)f):0)<<8);
+			retcol |= (((f < 255.0)?((int)(255.0-f)):255)<<16);
+			}
+		else if(y >= 0.25) {
+			f = (y - 0.25)*1024.0;		retcol |= 0x0000ff00L;
+			retcol |= (((f < 255.0)?((int)(255.0-f)):255)<<16);
+			}
+		else if(y >= 0.0) {
+			f = y*1024.0;			retcol |= 0x00ff0000L;
+			retcol |= (((f < 255.0)?((int)f):0)<<8);
+			}
+		else retcol = 0x00ff0000L;
+		break;
+	case 2:
+		retcol = 0x00000000L;
+		if(y >= 1.0) retcol = 0x00ffffffL;
+		else if(y >= (5.0/6.0)) {
+			f = (y - (5.0/6.0))*1536.0;	retcol |= 0x000000ffL;
+			retcol |= (((f < 255.0)?((int)f):0xff)<<8);
+			retcol |= (((f < 255.0)?((int)f):0xff)<<16);
+			}
+		else if(y >= (4.0/6.0)) {
+			f = (y - (4.0/6.0))*1536.0;	retcol |= 0x000000ffL;
+			retcol |= (((f < 255.0)?((int)(255.0-f)):0)<<8);
+			}
+		else if(y >= (3.0/6.0)) {
+			f = (y - (3.0/6.0))*1536.0;	retcol |= 0x0000ff00L;
+			retcol |= ((f < 255.0)?((int)f):0xff);
+			}
+		else if(y >= (2.0/6.0)) {
+			f = (y - (2.0/6.0))*1536.0;	retcol |= 0x0000ff00L;
+			retcol |= (((f < 255.0)?((int)(255.0-f)):0xff)<<16);
+			}
+		else if(y >= (1.0/6.0)) {
+			f = (y - (1.0/6.0))*1536.0;	retcol |= 0x00ff0000L;
+			retcol |= (((f < 255.0)?((int)f):0xff)<<8);
+			}
+		else if(y >= 0.0) {
+			f = y*1536.0;
+			retcol |= (((f < 255.0)?((int)f):0xff)<<16);
+			}
+		else retcol = 0x0L;
+		break;
+	case 3:
+		retcol = IpolCol(gCol_2, gCol_1, y);
+		}
+	if(gTrans & 0xff000000) {
+		retcol = (retcol & 0x00ffffff) | (gTrans & 0xff000000);
+		}
+	return retcol;
+}
+
 void
 Axis::SetTick(long idx, double val, DWORD flags, char *txt)
 {
@@ -1503,6 +1814,9 @@ Axis::CreateTicks()
 	if(axis->min == -HUGE_VAL || axis->min == HUGE_VAL) return;
 	if(axis->max == -HUGE_VAL || axis->max == HUGE_VAL) return;
 	if(axis->min >= axis->max) return;
+	if(Ticks) {
+		Undo.DropListGO(this, (GraphObj***)&Ticks, &NumTicks, 0L);
+		}
 	Command(CMD_FLUSH, 0L, 0L);
 	if((axis->flags & 0xf000) == AXIS_LOG) {	//log-axis
 		if(axis->Start > defs.min4log) tmp = log10(axis->Start);
@@ -1676,18 +1990,19 @@ Axis::GetValuePos(double val, double *fix, double *fiy, double *fiz, anyOutput *
 		op->fp2fip(&fdp, &fip);		*fix = fip.fx;			*fiy = fip.fy;
 		return bRet;
 		}
-	else if(type == 1 && fabs(flim[1].fx-flim[0].fx)>10.0) {	//x dominant
+	else if((type & 0x0f) == 1 && fabs(flim[1].fx-flim[0].fx)>10.0) {	//x dominant
 		tmp1 = (o->fx2fix(val)-flim[0].fx)/(flim[1].fx-flim[0].fx);
 		*fix = flim[0].fx - tmp1*(flim[0].fx - flim[1].fx);
 		*fiy = flim[0].fy - tmp1*(flim[0].fy - flim[1].fy);
 		}
-	else if(type == 2 && fabs(flim[1].fy-flim[0].fy)>10.0){		//y dominant
+	else if(((type & 0x0f) == 2  || (type &0x0f) == 4) && fabs(flim[1].fy-flim[0].fy)>10.0){	//y dominant
 		tmp1 = (o->fy2fiy(val)-flim[1].fy)/(flim[0].fy-flim[1].fy);
 		*fix = flim[1].fx + tmp1*(flim[0].fx - flim[1].fx);
 		*fiy = flim[1].fy + tmp1*(flim[0].fy - flim[1].fy);
 		}
-	if(tmp1 < -.005 || tmp1 > 1.005) return false;
-	return bRet;
+	else return false;
+	if(tmp1 > -1.0e-15 && tmp1 < 1.0 + 1.0e-15) return bRet;
+	return false;
 }
 
 void 
@@ -1886,3 +2201,42 @@ Axis::UpdateTicks()
 	SetSize(SIZE_TLB_YDIST, tlbdist.fy);
 	if(rT) delete(rT);	if(rL) delete(rL);	if(rMT) delete(rMT);
 }
+
+void
+Axis::GradientBar(anyOutput *o)
+{
+	FillDEF gf = {0, 0x0, 1.0, 0L, 0x0};
+	LineDEF gl = {0.0, 1.0, 0x0, 0x0};
+	POINT gpt[5];
+	double v_val, fix, fiy, fiz;
+	int i, iw;
+
+	if(!o || !Ticks || NumTicks < 2) return;
+	iw = o->un2ix(defs.GetSize(SIZE_AXIS_TICKS)*2);
+	GetValuePos(axis->min, &fix, &fiy, &fiz, o);
+	gpt[0].x = gpt[3].x = gpt[4].x = (iround(fix)-iw);
+	gpt[0].y = gpt[1].y = gpt[4].y = iround(fiy);
+	memcpy(&gradient_box, &gpt, sizeof(POINT)*5);		
+	gf.color = gf.color2 = gl.color = GradColor(axis->min);
+	for(i = 0; i < NumTicks; i++) if(Ticks[i]) {
+		if(GetValuePos(v_val = Ticks[i]->GetSize(SIZE_MINE), &fix, &fiy, &fiz, o)) {
+			gpt[1].x = gpt[2].x = iround(fix);		gpt[2].y = gpt[3].y = iround(fiy);
+			o->SetLine(&gl);	o->SetFill(&gf);	
+			if(gpt[1].y != gpt[2].y) o->oPolygon(gpt, 5, 0L);
+			gpt[0].x= gpt[3].x= gpt[4].x= (pts[1].x-iw);	gpt[0].y = gpt[1].y = gpt[4].y = gpt[2].y;
+			gf.color = gf.color2 = gl.color = GradColor(v_val);
+			}
+		}
+	if(GetValuePos(axis->max, &fix, &fiy, &fiz, o)) {
+		gpt[1].x = gpt[2].x = gradient_box[1].x = gradient_box[2].x = iround(fix);
+		gpt[2].y = gpt[3].y = gradient_box[2].y = gradient_box[3].y = iround(fiy);
+		o->SetLine(&gl);	o->SetFill(&gf);	
+		if(gpt[1].y != gpt[2].y) {
+			o->oPolygon(gpt, 5, 0L);
+			}
+		UpdateMinMaxRect(&rDims, gpt[0].x, gpt[1].y);
+		o->SetLine(&axline);
+		o->oPolyline(gradient_box, 5);
+		}
+	else o->SetLine(&axline);
+}
