@@ -1,4 +1,4 @@
-//ODbuttons.cpp, Copyright (c) 2001-2007 R.Lackner
+//ODbuttons.cpp, Copyright (c) 2001-2008 R.Lackner
 //Property dialogs for graphic objects
 //
 //    This file is part of RLPlot.
@@ -29,6 +29,27 @@ extern int ODtickstyle;
 extern int AxisTempl3D;
 extern Default defs;
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// utility draw base rectangle for OD-button
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+static void OD_BaseRect(anyOutput *o, int cmd, RECT *rec)
+{
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
+	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
+	POINT pts[5];
+
+	Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+	Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
+	o->SetLine(&Line);
+	pts[0].x = pts[3].x = pts[4].x = rec->left;
+	pts[0].y = pts[1].y = pts[4].y = rec->top;
+	pts[1].x = pts[2].x = rec->right-1;
+	pts[2].y = pts[3].y = rec->bottom-1;
+	o->oPolyline(pts, 5);				Line.color = 0x00000000L;
+	o->SetLine(&Line);				o->SetFill(&Fill);
+	o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Common code to modify drawing order in any dialog
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -110,6 +131,78 @@ int ExecDrawOrderButt(GraphObj *parent, GraphObj *obj, int id)
 	return id;
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// Execute polygon style as owner drawn buttons
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+void OD_PolygonStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
+		void *data, int id)
+{
+	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
+	POINT *pts;
+	int ix= (rec->left + rec->right)>>1, iy = (rec->top +rec->bottom)>>1, np;
+	long cp;
+	POINT tmppts[] = {{rec->left+15, iy}, {rec->left+15, iy-5}, {ix-5, rec->top+14},
+		{ix, rec->top+15}, {ix+10, rec->top+17}, {rec->right-7, rec->bottom-22},
+		{rec->right-15, rec->bottom-15}, {rec->right-23, rec->bottom-8},
+		{rec->left+15, iy+5}, {rec->left+15, iy}};
+	
+
+	if(!(pts=(POINT*)malloc(2*sizeof(POINT)*(rec->right-rec->left)))) return;
+	switch(cmd) {
+	case OD_DRAWNORMAL:
+	case OD_DRAWSELECTED:
+		OD_BaseRect(o, cmd, rec);
+		if(cmd == OD_DRAWSELECTED){
+			Fill.color = 0x0000ffffL;		o->SetFill(&Fill);
+			}
+
+		np = 0;
+		switch(id) {
+		case 201:
+			pts[np].x = rec->left+15;		pts[np++].y = iy;
+			pts[np].x = ix;				pts[np++].y = rec->top+15;
+			pts[np].x = rec->right-15;		pts[np++].y = rec->bottom-15;
+			break;
+		case 202:
+			pts[np].x = rec->left+15;		pts[np++].y = iy;
+			pts[np].x = ix;				pts[np++].y = iy;
+			pts[np].x = ix;				pts[np++].y = rec->top+15;
+			pts[np].x = rec->right-15;		pts[np++].y = rec->top+15;
+			pts[np].x = rec->right-15;		pts[np++].y = rec->bottom-15;
+			pts[np].x = rec->left+15;		pts[np++].y = rec->bottom-15;
+			break;
+		case 203:
+			pts[np].x = rec->left+15;		pts[np++].y = iy;
+			pts[np].x = rec->left+15;		pts[np++].y = rec->top+15;
+			pts[np].x = ix;				pts[np++].y = rec->top+15;
+			pts[np].x = ix;				pts[np++].y = rec->bottom-15;
+			pts[np].x = rec->right-15;		pts[np++].y = rec->bottom-15;
+			pts[np].x = rec->right-15;		pts[np++].y = iy;
+			break;
+		case 213:
+			cp = 0;
+			DrawBezier(&cp, pts, tmppts[0], tmppts[1], tmppts[2], tmppts[3], 0);
+			DrawBezier(&cp, pts, tmppts[3], tmppts[4], tmppts[5], tmppts[6], 0);
+			DrawBezier(&cp, pts, tmppts[6], tmppts[7], tmppts[8], tmppts[9], 0);
+			np = (int)cp;
+			break;
+			}
+		if(np) o->oPolygon(pts, np);
+		switch(id) {
+		case 201:	case 202:	case 203:	case 213:
+			Fill.color = (cmd == OD_DRAWSELECTED) ? 0x000000ffL : 0x00ffffffL;
+			o->SetFill(&Fill);
+			o->oCircle(ix-2, rec->top+13, ix+3, rec->top+18);
+			o->oCircle(rec->left+13, iy-2, rec->left+18, iy+3);
+			o->oCircle(rec->right-14, rec->bottom-14, rec->right-17, rec->bottom-17);
+			break;
+			}
+		o->UpdateRect(rec, false);
+		free(pts);
+		break;
+		}
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Execute line style as owner drawn buttons
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -121,7 +214,7 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	POINT *pts;
 	int i, ix, iy, np;
 
-	if(!(pts=(POINT*)malloc(sizeof(POINT)*(rec->right-rec->left)))) return;
+	if(!(pts=(POINT*)malloc(2*sizeof(POINT)*(rec->right-rec->left)))) return;
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
@@ -253,35 +346,58 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		}
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// Execute bubble style as owner drawn buttons
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+void OD_BubbleTempl(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id)
+{
+	FillDEF Fill = {FILL_NONE, 0x00c0ffffL, 1.0, 0L};
+	POINT pts[3];
+	int ix, iy;
+
+	switch(cmd) {
+	case OD_DRAWNORMAL:
+	case OD_DRAWSELECTED:
+		ix = (rec->left + rec->right)/2;
+		iy = (rec->top +rec->bottom)/2;
+		pts[0].x = ix-10;	pts[2].x = ix+10;	pts[1].x = ix;
+		OD_BaseRect(o, cmd, rec);
+		if(cmd == OD_DRAWSELECTED) o->SetFill(&Fill);
+		switch(id) {
+		case 109:	case 201:
+			o->oCircle(ix-10, iy-10, ix+10, iy+10);
+			break;
+		case 110:	case 202:
+			o->oRectangle(ix-10, iy-10, ix+10, iy+10);
+			break;
+		case 111:	case 203:
+			pts[0].y = pts[2].y = iy + 9;		pts[1].y = iy - 11;
+			o->oPolygon(pts, 3);
+			break;
+		case 112:	case 204:
+			pts[0].y = pts[2].y = iy - 9;		pts[1].y = iy + 11;
+			o->oPolygon(pts, 3);
+			break;
+			}
+		o->UpdateRect(rec, false);
+		}
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Execute error bar style as owner drawn buttons for the error bar dialog
 // and in the scatterplot dialog
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
-		void *data, int id)
+void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id)
 {
-	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[6];
 	int ix, iy;
 
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		switch(id) {
 		case 500:
 			pts[2].x = pts[3].x = ix;
@@ -324,28 +440,15 @@ void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 void OD_WhiskerTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[6];
 	int ix, iy;
 
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		switch(id) {
 		case 500:
 			pts[2].x = pts[3].x = ix;
@@ -463,13 +566,11 @@ void OD_PieTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
 	LineDEF Line = {.0, 1.0, 0x0L, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	FillDEF FillR = {FILL_NONE, 0x000000ffL, 1.0, 0L};
 	FillDEF FillG = {FILL_NONE, 0x0000ff00L, 1.0, 0L};
 	FillDEF FillB = {FILL_NONE, 0x00ff0000L, 1.0, 0L};
 	double angels1[]={90.0, 45.0, -45.0, 90.0};
 	double angels2[]={180, 157.5, 112.5, 0.0};
-	POINT pts[12];
 	int ix, iy;
 	double r, *ang = angels1;
 	segment *seg = 0L;
@@ -478,20 +579,9 @@ void OD_PieTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		switch(id) {
 		case 401:		case 411:
 			ang = angels2;
@@ -574,20 +664,9 @@ void OD_BreakTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)>>1;
 		iy = (rec->top +rec->bottom)>>1;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		pts[0].x = pts[1].x = ix;
 		pts[0].y = rec->top +5;		pts[1].y = iy-3;
 		o->oSolidLine(pts);
@@ -626,16 +705,16 @@ void OD_BreakTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Execute plot selection templates as owner drawn button
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-void UtilBarDraw(POINT *pts, int x, int y1, int y2, anyOutput *o)
+static void UtilBarDraw(POINT *pts, int x, int y1, int y2, anyOutput *o)
 {
 	pts[1].x = pts[0].x = pts[4].x = x;
-	pts[0].y = y1;				pts[1].y = y2;
+	pts[0].y = y1;			pts[1].y = y2;
 	pts[2].y = pts[1].y-1;		pts[2].x = pts[3].x = pts[0].x-3;	
 	pts[3].y = pts[0].y-1;		pts[4].y = pts[0].y;
 	o->oPolygon(pts, 5);
-	pts[2].x += 5;				pts[3].x += 5;
+	pts[2].x += 5;			pts[3].x += 5;
 	o->oPolygon(pts, 5);
-	pts[1].x -= 3;				pts[1].y = pts[0].y-1;
+	pts[1].x -= 3;			pts[1].y = pts[0].y-1;
 	pts[2].x = pts[0].x-2;		pts[2].y = pts[0].y-2;
 	o->oPolygon(pts, 5);
 }
@@ -647,7 +726,6 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	LineDEF rLine = {.0, 1.0, 0x00000080L, 0x0L};
 	LineDEF bLine = {.0, 1.0, 0x00e00000L, 0x0L};
 	LineDEF gLine = {.0, 1.0, 0x0000e000L, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	FillDEF FillR = {FILL_NONE, 0x000000ffL, 1.0, 0L};
 	FillDEF FillG = {FILL_NONE, 0x0000ff00L, 1.0, 0L};
 	FillDEF FillB = {FILL_NONE, 0x00ff0000L, 1.0, 0L};
@@ -670,20 +748,13 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			OD_AxisTempl3D(cmd, par, rec, o, data, 410+AxisTempl3D);
 			break;
 		default:
-			Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-			Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
-			o->SetLine(&Line);
-			pts[0].x = pts[3].x = pts[4].x = rec->left;
-			pts[0].y = pts[1].y = pts[4].y = rec->top;
-			pts[1].x = pts[2].x = rec->right-1;
-			pts[2].y = pts[3].y = rec->bottom-1;
-			o->oPolyline(pts, 5);
-			Line.color = 0x00000000L;
-			o->SetLine(&Line);
-			o->SetFill(&Fill);
-			o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+			OD_BaseRect(o, cmd, rec);
 			break;
 			}
+		if(cmd != OD_DRAWSELECTED) {
+			FillR.color |= 0x00808080L;		FillG.color |= 0x00808080L;
+			FillB.color |= 0x00808080L;		FillY.color |= 0x00808080L;
+			}
 		switch(id) {
 		case 500:
 		case 501:
@@ -1179,6 +1250,39 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			pts[1].x = ix+12;		pts[1].y = iy+4;		o->oPolygon(pts, 3, 0L);
 			pts[0].x = ix+16;		pts[0].y = iy+12;		o->oPolygon(pts, 3, 0L);
 			break;
+		case 568:
+			if(cmd == OD_DRAWSELECTED) {
+				FillY.color |= 0x00808080L;			FillG.color |= 0x00808080L;
+				FillR.color |= 0x00808080L;			FillB.color |= 0x00808080L;
+				o->SetFill(&FillY);
+				pts[0].x = pts[3].x = rec->left+3;		pts[0].y = pts[1].y = rec->top+3;
+				pts[1].x = pts[2].x = rec->right-3;		pts[2].y = pts[3].y = rec->bottom-3;
+				o->oPolygon(pts, 4);
+				}
+			o->SetLine(cmd == OD_DRAWSELECTED ? &bLine : &Line);
+			if(cmd == OD_DRAWSELECTED) o->SetFill(&FillG);
+			pts[0].x = pts[1].x = rec->left+15;				pts[0].y = rec->top+3;
+			pts[1].y = rec->top+5;		pts[2].x = rec->left+12;	pts[2].y = rec->top+12;
+			pts[3].x = rec->left+7;		pts[3].y = pts[4].y = rec->top+15;
+			pts[4].x = rec->left+3;		pts[5].x = rec->left+3;		pts[5].y = rec->top+3;
+			o->oPolygon(pts, 6);
+			pts[0].x = pts[1].x = ix;	pts[2].x = ix+6;		pts[3].x = rec->right-7;
+			pts[4].x = rec->right-3;	pts[5].x = rec->right-3;	o->oPolygon(pts, 6);
+			pts[0].y = rec->bottom-3;	pts[1].y = rec->bottom-9;	pts[2].y = iy+3;
+			pts[3].x = rec->right-9;	pts[3].y = pts[4].y = iy-3;	pts[5].y = rec->bottom-3;
+			o->oPolygon(pts, 6);
+			pts[0].x = pts[1].x = rec->right-15;				pts[0].y = rec->bottom-3;
+			pts[1].y = rec->bottom-5;	pts[2].x = rec->right-12;	pts[2].y = rec->bottom-10;
+			pts[3].x = rec->right-7;	pts[3].y = pts[4].y = rec->bottom-15;
+			pts[4].x = rec->right-3;	pts[5].x = rec->right-3;	pts[5].y = rec->bottom-3;
+			if(cmd == OD_DRAWSELECTED) o->SetFill(&FillB);			o->oPolygon(pts, 6);
+			pts[0].x = pts[1].x = rec->left+15;				pts[0].y = rec->bottom-3;
+			pts[1].y = rec->bottom-5;	pts[2].x = rec->left+12;	pts[2].y = iy+5;
+			pts[3].x = rec->left+7;		pts[3].y = pts[4].y = iy-3;	pts[4].x = rec->left+3;
+			pts[5].x = rec->left+3;		if(cmd == OD_DRAWSELECTED) o->SetFill(&FillG);
+			o->oPolygon(pts, 6);
+			if(cmd == OD_DRAWSELECTED) o->SetFill(&FillR);	o->oCircle(ix-9, iy-9, ix+1, iy+3);
+			break;
 			}
 		o->UpdateRect(rec, false);
 		break;
@@ -1193,25 +1297,13 @@ void OD_AxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 {
 	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	LineDEF Grid = {0.0, 1.0, 0x00c0c0c0, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[5];
 	int i, ty, tx, sx;
 
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		tx = ty = 0;
 		switch(id) {
 		case 310:
@@ -1319,17 +1411,7 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
-		o->SetLine(&Line);
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);						o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		Line.color = cmd == OD_DRAWSELECTED ? 0x00c0c0c0L : 0x0L;
 		o->SetLine(&Line);						o->SetFill(&Fill);
 		switch(id) {
@@ -1378,28 +1460,15 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
-	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[5];
 	int i, ix, iy, step, d1, d2;
 
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
-		o->SetLine(&Line);
 		ix = (rec->right + rec->left)>>1;
 		iy = (rec->bottom + rec->top)>>1;
-		pts[0].x = pts[3].x = pts[4].x = rec->left;
-		pts[0].y = pts[1].y = pts[4].y = rec->top;
-		pts[1].x = pts[2].x = rec->right-1;
-		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
-		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		OD_BaseRect(o, cmd, rec);
 		d1 = d2 = 0;
 		switch(id) {
 		case 201:			d1 = -6;				d2 = -2;
@@ -1442,8 +1511,7 @@ void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // 3D Plot: Execute axis templates for new axis as owner drawn buttons
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-void OD_NewAxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
-		void *data, int id)
+void OD_NewAxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id)
 {
 	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
@@ -1464,7 +1532,7 @@ void OD_NewAxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 		spts[0].y = spts[1].y = spts[4].y = rec->top;
 		spts[1].x = spts[2].x = rec->right-1;	spts[2].y = spts[3].y = rec->bottom-1;
 		o->oPolyline(spts, 5);					Line.color = 0x00000000L;
-		o->SetLine(&Line);						o->SetFill(&Fill);
+		o->SetLine(&Line);					o->SetFill(&Fill);
 		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
 		spts[0].x = spts[6].x = spts[8].x = spts[13].x = spts[18].x = spts[19].x = rec->left+20;		
 		spts[0].y = spts[13].y = spts[18].y = rec->bottom-14;
