@@ -1,4 +1,4 @@
-//Utils.cpp, Copyright (c) 2000-2007 R.Lackner
+//Utils.cpp, Copyright (c) 2000-2008 R.Lackner
 //Collection of utility functions and classes for RLPlot
 //
 //    This file is part of RLPlot.
@@ -871,7 +871,7 @@ bool IsInPolygon3D(double x, double y, POINT3D *pts, int cp, int *us, int *ls)
 				}
 			}
 		}
-	//return an index to the bracketing sgments of the polygon
+	//return an index to the bracketing segments of the polygon
 	if(us) *us = idx1;			if(ls) *ls = idx2;
 	return((tmp1 & 0x1) && (tmp2 & 0x1));
 }
@@ -938,10 +938,10 @@ void AddToPolygon(long *cp, POINT *pts, POINT *np)
 
 //----------------------------------------------------------------------------
 // create a Bezier polygon
-void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth)
-{
 #define MIN_SEG 11
 #define MAX_DEPTH 5
+void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth)
+{
 	int i;
 	POINT np, p01, p12, p23, p012, p123, p0123;
 	POINT *ap[] = {&p0, &p1, &p2, &p3};
@@ -957,28 +957,83 @@ void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, in
 	else if(depth == 1) for(i=0; i < 4; i++) {
 		(*ap[i]).x <<= 2;			(*ap[i]).y <<= 2;
 		}
-	p01.x = (p0.x + p1.x) >> 1;				p01.y = (p0.y + p1.y) >> 1;
-	p12.x = (p1.x + p2.x) >> 1;				p12.y = (p1.y + p2.y) >> 1;
-	p23.x = (p2.x + p3.x) >> 1;				p23.y = (p2.y + p3.y) >> 1;
+	p01.x = (p0.x + p1.x) >> 1;			p01.y = (p0.y + p1.y) >> 1;
+	p12.x = (p1.x + p2.x) >> 1;			p12.y = (p1.y + p2.y) >> 1;
+	p23.x = (p2.x + p3.x) >> 1;			p23.y = (p2.y + p3.y) >> 1;
 	p012.x = (p01.x + p12.x) >> 1;			p012.y = (p01.y + p12.y) >> 1;
 	p123.x = (p12.x + p23.x) >> 1;			p123.y = (p12.y + p23.y) >> 1;
 	p0123.x = (p012.x + p123.x) >> 1;		p0123.y = (p012.y + p123.y) >> 1;
 	if(abs(p0.x - p0123.x)> MIN_SEG || abs(p0.y - p0123.y)> MIN_SEG) {
-		DrawBezier(cp, pts, p0, p01, p012, p0123, depth);		//recursion: refine
+		DrawBezier(cp, pts, p0, p01, p012, p0123, depth);	//recursion: refine
 		}
 	else {
 		DrawBezier(cp, pts, p0, p01, p012, p0123, MAX_DEPTH);	//recursion: store data
 		}
 	if(abs(p3.x - p0123.x)> MIN_SEG || abs(p3.y - p0123.y)> MIN_SEG) {
-		DrawBezier(cp, pts, p0123, p123, p23, p3, depth);		//recursion: refine
+		DrawBezier(cp, pts, p0123, p123, p23, p3, depth);	//recursion: refine
 		}
 	else {
 		DrawBezier(cp, pts, p0123, p123, p23, p3, MAX_DEPTH);	//recursion: store data
 		}
+}
 #undef MAX_DEPTH
 #undef MIN_SEG
+
+// create a Bezier polygon clipped to rectangle
+static RECT BezClipRec;
+#define MIN_SEGCLP 5
+#define MAX_DEPTHCLP 6
+static void DrawBezierClip(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth)
+{
+	int i;
+	POINT np, p01, p12, p23, p012, p123, p0123;
+	POINT *ap[] = {&p0, &p1, &p2, &p3};
+
+	depth++;
+	if(depth > MAX_DEPTHCLP) {
+		for(i= 0; i < 4; i++) {
+			np.x = (*ap[i]).x >> 2;		np.y = (*ap[i]).y >> 2;
+			if(np.x < BezClipRec.left) np.x = BezClipRec.left;
+			if(np.x > BezClipRec.right) np.x = BezClipRec.right;
+			if(np.y < BezClipRec.top) np.y = BezClipRec.top;
+			if(np.y > BezClipRec.bottom) np.y = BezClipRec.bottom;
+			AddToPolygon(cp, pts, &np);
+			}
+		return;
+		}
+	else if(depth == 1) for(i=0; i < 4; i++) {
+		(*ap[i]).x <<= 2;			(*ap[i]).y <<= 2;
+		}
+	p01.x = (p0.x + p1.x) >> 1;			p01.y = (p0.y + p1.y) >> 1;
+	p12.x = (p1.x + p2.x) >> 1;			p12.y = (p1.y + p2.y) >> 1;
+	p23.x = (p2.x + p3.x) >> 1;			p23.y = (p2.y + p3.y) >> 1;
+	p012.x = (p01.x + p12.x) >> 1;			p012.y = (p01.y + p12.y) >> 1;
+	p123.x = (p12.x + p23.x) >> 1;			p123.y = (p12.y + p23.y) >> 1;
+	p0123.x = (p012.x + p123.x) >> 1;		p0123.y = (p012.y + p123.y) >> 1;
+	if((abs(p0.x - p0123.x)> MIN_SEGCLP || abs(p0.y - p0123.y)> MIN_SEGCLP)) {
+		DrawBezierClip(cp, pts, p0, p01, p012, p0123, depth);		//recursion: refine
+		}
+	else {
+		DrawBezierClip(cp, pts, p0, p01, p012, p0123, MAX_DEPTHCLP);	//store data
+		}
+	if((abs(p3.x - p0123.x)> MIN_SEGCLP || abs(p3.y - p0123.y)> MIN_SEGCLP)) {
+		DrawBezierClip(cp, pts, p0123, p123, p23, p3, depth);		//recursion: refine
+		}
+	else {
+		DrawBezierClip(cp, pts, p0123, p123, p23, p3, MAX_DEPTHCLP);	//store data
+		}
 }
 
+void ClipBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, POINT *clp1, POINT *clp2)
+{
+	if(clp1 && clp2){
+		SetMinMaxRect(&BezClipRec, clp1->x, clp1->y, clp2->x, clp2->y);
+		}
+	if(cp && pts) DrawBezierClip(cp, pts, p0, p1, p2, p3, 0);
+}
+#undef MIN_SEGCLP
+#undef MAX_DEPTHCLP
+
 //----------------------------------------------------------------------------
 // create a Bezier spline through data points
 static void ipol_curve(lfPOINT *p1, lfPOINT *p2, lfPOINT *p3, lfPOINT *cp1, lfPOINT *cp2)
@@ -1113,37 +1168,6 @@ POINT *MakeArc(int ix, int iy, int r, int qad, long *npts)
 	return pts;
 }
 
-//----------------------------------------------------------------------------
-// display a marked polygon by displaying it with complement colors
-//----------------------------------------------------------------------------
-void InvertPolygon(POINT *pts, int nPts, LineDEF *Line, FillDEF *Fill, RECT *rc, 
-	anyOutput *o, bool mark)
-{
-	LineDEF *FillLine;
-
-	if(!Line || !Fill || !rc || !o) return;
-	FillLine = Fill->hatch;
-	if(mark) {
-		if(FillLine)FillLine->color ^= 0x00ffffffL;
-		Fill->color ^= 0x00ffffffL;
-		Line->color ^= 0x00ffffffL;
-		o->SetLine(Line);
-		o->SetFill(Fill);
-		o->oPolygon(pts, nPts);
-		Fill->color ^= 0x00ffffffL;
-		if(FillLine)FillLine->color ^= 0x00ffffffL;
-		Line->color ^= 0x00ffffffL;
-		InvertLine(pts, nPts, Line, 0L, o, mark);
-		}
-	else {
-		InvertLine(pts, nPts, Line, 0L, o, mark);
-		o->SetLine(Line);
-		o->SetFill(Fill);
-		o->oPolygon(pts, nPts);
-		}
-	if(rc) o->UpdateRect(rc, false);
-}
-
 //----------------------------------------------------------------------------
 // display a marked line using complement colors
 //----------------------------------------------------------------------------
@@ -1156,10 +1180,10 @@ void InvertLine(POINT *pts, int nPts, LineDEF *Line, RECT *rc,
 	memcpy(&OldLine, Line, sizeof(LineDEF));
 	if(OldLine.width <= 0.0001) OldLine.width = 0.0001;
 	for(i = 0; o->un2fiy(OldLine.width) < 1.0 && i < 50; i++) OldLine.width *= 2.0;
-	OldLine.color = mark ? Line->color : 0x00ffffffL;
+	OldLine.color = mark ? (Line->color & 0x00ffffffL) : 0x00ffffffL;
 	OldLine.width *= 3.0;				o->SetLine(&OldLine);
 	o->oPolyline(pts, nPts);			OldLine.width = Line->width;
-	OldLine.color = mark ? Line->color ^ 0x00ffffffL : Line->color;
+	OldLine.color = mark ? (Line->color & 0x00ffffffL) ^ 0x00ffffffL : (Line->color & 0x00ffffff);
 	o->SetLine(&OldLine);				o->oPolyline(pts, nPts);
 	if(rc) o->UpdateRect(rc, false);
 }
@@ -1189,12 +1213,12 @@ DWORD IpolCol(DWORD color1, DWORD color2, double fact)
 	int i;
 
 	col1 = color1;	col2 = color2;	col3 = 0x0L;
-	for(i = 0; i < 3; i++) {
-		c1 = iround(fabs(((col1 & 0xff0000)>>16) * fact));
-		c1 += iround(fabs(((col2 & 0xff0000)>>16) *(1.0-fact)));
+	for(i = 0; i < 4; i++) {
+		c1 = iround(fabs((((col1 & 0xff000000)>>24) & 0xff) * fact));
+		c1 += iround(fabs((((col2 & 0xff000000)>>24) & 0xff) *(1.0-fact)));
 		col3 |= c1 < 0xff ? c1 : 0xff;
 		col1 <<= 8;	col2 <<= 8; 
-		if(i < 2) col3 <<= 8;
+		if(i < 3) col3 <<= 8;
 		}
 	return col3;
 }
@@ -1379,6 +1403,7 @@ void DeleteGO(GraphObj *go)
 	case GO_BEZIER:			delete((Bezier*)go);		break;
 	case GO_TEXTFRAME:		delete((TextFrame*)go);		break;
 	case GO_NORMQUANT:		delete((NormQuant*)go);		break;
+	case GO_CONTOUR:		delete((ContourPlot*)go);	break;
 	default:
 #ifdef USE_WIN_SECURE
 		sprintf_s(TmpTxt, TMP_TXT_SIZE, "Cannot delete Object\nwith Id %ld", go->Id);
@@ -2108,7 +2133,7 @@ bool ShadowPolygon(POINT *p1, POINT *p2, POINT *tp, int ntp, POINT *pts, long *c
 		}
 }
 
-//find sepment which is closest to point
+//find segment which is closest to point
 int FindClosestSeg(POINT3D *pg, int npg, int x, int y, int start)
 {
 	int i, i1, j, tmp, idx = -2, dx, dy, d, dist = 10000;
@@ -2158,7 +2183,7 @@ bool AddShadowPolygon(POINT3D *pnt, POINT3D *ep, int cidx) {
 	d = ((d1 = (ep->x - pnt->x))*d1);
 	d += ((d1 = (ep->y - pnt->y))*d1);
 	if(d < 4) {			//propably too close
-		if(d) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
+		if(d && ppg_par) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
 		return true;	//connect
 		}
 	lim.x = ep->x;		lim.y = ep->y;
@@ -2208,7 +2233,7 @@ bool AddShadowPolygon(POINT3D *pnt, POINT3D *ep, int cidx) {
 		tp = tp2;	ntp = ntp2;
 		}
 	else if (d1 == d2 && d1 < 5) {			//ambiguous result: connect stright 
-		if(d) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
+		if(d && ppg_par) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
 		}
 	else {							//no valid solution;
 		if(cidx >= 0) return AddShadowPolygon(pnt, ep, -2);
@@ -2232,7 +2257,7 @@ bool AddShadowPolygon(POINT3D *pnt, POINT3D *ep, int cidx) {
 				IsCloseToPL(np, ppg_mask, ppg_nmask)){
 				if(ppg_vec) {					//valid plane eqation
 					nep.z = iround((nep.x * ppg_vec[0] + nep.y * ppg_vec[1] - ppg_vec[3])/ppg_vec[2]);
-					ppg_par->Command(CMD_ADDTOLINE, &nep, 0L);
+					if(ppg_par) ppg_par->Command(CMD_ADDTOLINE, &nep, 0L);
 					}
 				else if(IsInPolygon3D(nep.x, nep.y, ppg_act, ppg_nact, &us, &ls)) {
 					if(us == ls) if(ls){	//point is on the line
@@ -2255,7 +2280,7 @@ bool AddShadowPolygon(POINT3D *pnt, POINT3D *ep, int cidx) {
 							nep.z = (z1 + z2)>>1;					//impropable
 							}
 						}
-					ppg_par->Command(CMD_ADDTOLINE, &nep, 0L);
+					if(ppg_par) ppg_par->Command(CMD_ADDTOLINE, &nep, 0L);
 					}
 				else {
 					//point is inside by one algorithm but outside with another
@@ -2263,7 +2288,7 @@ bool AddShadowPolygon(POINT3D *pnt, POINT3D *ep, int cidx) {
 					}
 				}
 			}
-		if(nep.x != ep->x || nep.y != ep->y) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
+		if(ppg_par && (nep.x != ep->x || nep.y != ep->y)) ppg_par->Command(CMD_ADDTOLINE, ep, 0L);
 		}
 	free(pg1);	free(tp1);	free(tp2);
 	return bRet;
@@ -2314,16 +2339,18 @@ bool CuttingEdge(POINT3D* pt, POINT3D* np)
 
 //come here from clip_line_3D to process changes in visibility when
 //   clipping one polygon with another
-void proc_polygon(int vis, POINT3D *pnt, POINT3D *last)
+void proc_polygon(int vis1, POINT3D *pnt, POINT3D *last)
 {
 	static POINT3D np, lp;
 	long d, d1;
+	int vis = vis1;
 	bool spg_valid;
 
 	switch(ppg_level){
 	case 0:						//searching first visible point of polygon
 		if(vis == 3) vis = 1;						//on line is visible
 		if(!ppg_vis && vis && !ppg_nowvis){			//found it !
+			if(ppg_par && (vis1 == 3 || ppg_vis == 3)) ppg_par->Command(CMD_SIGNAL_POL, pnt, 0L);
 			ppg_nowvis = true;			ppg_first.x = pnt->x;
 			ppg_first.y = pnt->y;		ppg_first.z = pnt->z;
 			ppg_reason = vis;
@@ -2336,41 +2363,48 @@ void proc_polygon(int vis, POINT3D *pnt, POINT3D *last)
 		ppg_vis = vis;
 		break;
 	case 1:
-		if(vis == 3) vis = 1;						//on line: visible
+		if(vis == 3) {						//on line: visible
+			vis = 1;
+			}
 		if(ppg_first.x < 0 && ppg_first.y < 0 && vis) {
 			memcpy(&ppg_first, pnt, sizeof(POINT3D));
 			ppg_firstvis = vis;			lp.x = pnt->x;
 			lp.y = pnt->y;				lp.z = pnt->z;
-			ppg_par->Command(CMD_STARTLINE, pnt, 0L);
+			if(ppg_par) ppg_par->Command(CMD_STARTLINE, pnt, 0L);
 			}
 		else if (ppg_vis) {			//leaving visibility or continue
 			spg_valid = false;
 			if(lp.x == pnt->x && lp.y == pnt->y && lp.z == pnt->z) break;
-			if(vis) ppg_par->Command(CMD_ADDTOLINE, pnt, 0L);
-			else ppg_par->Command(CMD_ADDTOLINE, last, 0L);
+			if(vis1 == 3 && ppg_par) ppg_par->Command(CMD_SIGNAL_POL, pnt, 0L);
+			if(vis && ppg_par) ppg_par->Command(CMD_ADDTOLINE, pnt, 0L);
+			else if(ppg_par) ppg_par->Command(CMD_ADDTOLINE, last, 0L);
 			if(!vis){				//leaving visibility
 				ppg_vis = test_plane(last->x, last->y, last->z);
+				if(ppg_vis == 3 && ppg_par) ppg_par->Command(CMD_SIGNAL_POL, pnt, 0L);
 				if(ppg_vis == 3) ppg_vis = 1;
 				if(ppg_firstvis == 1 && ppg_vis == 1) {
 					//from below surface to below surface
-					spg_valid = AddShadowPolygon(last, &ppg_first, -2);
+					if(!(spg_valid = AddShadowPolygon(last, &ppg_first, -2)))
+						if(ppg_par) spg_valid = ppg_par->Command(CMD_ADDTOLINE, pnt, 0L);
 					}
 				else if(ppg_firstvis == 1 && ppg_vis == 2) {
 					//from below surface enter inside surface
 					if(CuttingEdge(last, &np)){
-						ppg_par->Command(CMD_ADDTOLINE, &np, 0L);
+						if(ppg_par)ppg_par->Command(CMD_ADDTOLINE, &np, 0L);
 						spg_valid = AddShadowPolygon(&np, &ppg_first, seg_x_seg);
 						}
-					else spg_valid = AddShadowPolygon(last, &ppg_first, seg_x_seg);
+					else if(!(spg_valid = AddShadowPolygon(last, &ppg_first, seg_x_seg)))
+						if(ppg_par) spg_valid = ppg_par->Command(CMD_ADDTOLINE, pnt, 0L);
 					}
-				else if(ppg_firstvis == 2 && ppg_vis == 1) {
+				else if(ppg_firstvis == 2 && ppg_vis == 1 && ppg_par) {
 					//from inside surface to below surface
 					if(CuttingEdge(&ppg_first, &np)){
 						if(!(spg_valid = AddShadowPolygon(last, &np, seg_x_seg)))
-							ppg_par->Command(CMD_REQ_POINT, &ppg_first, 0L);
+							spg_valid = ppg_par->Command(CMD_REQ_POINT, &ppg_first, 0L);
 						ppg_par->Command(CMD_ADDTOLINE, &np, 0L);
 						}
-					else spg_valid = AddShadowPolygon(last, &ppg_first, seg_x_seg);
+					else if(!(spg_valid = AddShadowPolygon(last, &ppg_first, seg_x_seg)))
+						spg_valid = ppg_par->Command(CMD_ADDTOLINE, pnt, 0L);
 					}
 				else if(ppg_firstvis == 2 && ppg_vis == 2) {
 					//from inside surface to inside surface
@@ -2460,7 +2494,7 @@ void clip_plane_plane(GraphObj *par, POINT3D *pg1, int n1, double *v1, POINT3D *
 	POINT3D sp;
 
 	nclp_pg = n2;	clp_pg = pg2;	ppg_level = 0;	ppg_nowvis = false;
-	ppg_reason = 0;	vclp_pg = v2;
+	ppg_reason = 0;	vclp_pg = v2;	ppg_par = par;
     ppg_vis = test_plane(pg1[0].x, pg1[0].y, pg1[0].z);
 	ppg_act = pg1;	ppg_nact = n1;	ppg_vec = v1;
 	for(i = 1; i < n1 && !ppg_nowvis; i++){		//assume first pnt == last pnt
@@ -2472,7 +2506,7 @@ void clip_plane_plane(GraphObj *par, POINT3D *pg1, int n1, double *v1, POINT3D *
 		}
 	if(ppg_nowvis) {							//clip this polygon
 		ppg_mask = mask;		ppg_nmask = nmask;
-		ppg_level = 1;			ppg_par = par;
+		ppg_level = 1;
 		sp.x = ppg_first.x;		sp.y = ppg_first.y;
 		sp.z = ppg_first.z;		ppg_first.x = ppg_first.y = ppg_first.z = -1;
 		ppg_vis = test_plane(sp.x, sp.y, sp.z);
