@@ -1,4 +1,4 @@
-//WinSpec.cpp, Copyright (c) 2000-2007 R.Lackner
+//WinSpec.cpp, Copyright (c) 2000-2008 R.Lackner
 //the entire code of this module is highly specific to Windows!
 //
 //    This file is part of RLPlot.
@@ -46,6 +46,7 @@ const char name[] = "RLPLOT1";
 
 static unsigned int cf_rlpobj = RegisterClipboardFormat("rlp_obj");
 static unsigned int cf_rlpxml = RegisterClipboardFormat("rlp_xml");
+static char *ShellCmd;
 
 long FAR PASCAL WndProc(HWND, UINT, UINT, LONG);
 PrintWin *Printer = 0L;
@@ -327,6 +328,7 @@ void ShowTextCursor(anyOutput *out, RECT *disp, DWORD color)
 {
 	HWND wnd = GetFocus();
 
+	if(!out || (out->OC_type & 0xff) != OC_BITMAP) return;
 	cTxtCur = color;			HideTextCursor();
 	oTxtCur = out;				bSuspend = false;
 	iTxtCurCount = -2;
@@ -354,6 +356,7 @@ void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec)
 {
 	int i;
 
+	if(!out || (out->OC_type & 0xff) != OC_BITMAP) return;
 	HideCopyMark();			bSuspend = false;
 	if(!out || !mrk || !nRec) return;
 	oCopyMark = out;
@@ -369,7 +372,7 @@ void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec)
 
 void InvalidateOutput(anyOutput *o)
 {
-	if(!o) return;
+	if(!o || (o->OC_type & 0xff) != OC_BITMAP) return;
 	if(o == oCopyMark) {
 		oCopyMark = 0L;
 		if(bmCopyMark) delete bmCopyMark;
@@ -382,7 +385,7 @@ void InvalidateOutput(anyOutput *o)
 
 void SuspendAnimation(anyOutput *o, bool bSusp)
 {
-	if(!o) return;
+	if(!o || (o->OC_type & 0xff) != OC_BITMAP) return;
 	if(!bSusp) bSuspend = false;
 	else {
 		if(o == oCopyMark) bSuspend = bSusp;
@@ -401,7 +404,7 @@ LRESULT FAR PASCAL TimerWndProc(HWND hwnd, UINT message, UINT wParam, LONG lPara
 	switch(message) {
 	case WM_TIMER:
 		if(bSuspend) return 0;
-		if(bmCopyMark && oCopyMark) {
+		if(bmCopyMark && oCopyMark && (oCopyMark->OC_type & 0xff) == OC_BITMAP) {
 			bmCopyMark->CopyBitmap(0, 0, oCopyMark, rCopyMark.left, rCopyMark.top, 
 				rCopyMark.right - rCopyMark.left, rCopyMark.bottom - rCopyMark.top, false);
 			bmCopyMark->SetLine(&liCopyMark1);
@@ -602,16 +605,52 @@ bool com_oTextOutW(int x, int y, w_char *txt, int cb, HFONT *hFont, HDC *dc,
 	TextDEF *td, anyOutput *o)
 {
 	XFORM xf;
-	int ix, iy, w, h, dtflags;
+	int ix, iy, w, h, dtflags, rx, ry;
+	double si, csi;
 	RECT dtrc;
+	fRECT rc;
+	TextDEF ttd;
 
 
 	if(!*hFont || !txt || !txt[0]) return false;
 	if(cb < 1) cb = (int)wcslen(txt);
-	SelectObject(*dc, *hFont);				SetTextColor(*dc, td->ColTxt);
-	SetBkColor(*dc, td->ColBg);				SetBkMode(*dc, td->Mode ? TRANSPARENT : OPAQUE);
-	ix = iy = 0;							SetTextAlign(*dc, TA_LEFT | TA_TOP);
-	if(o->OC_type == OC_HIMETRIC) {
+	//test for transparency
+	if(((td->ColTxt & 0xff000000L) || (td->ColBg & 0xff000000L)) && (o->OC_type & 0xff) == OC_BITMAP) {
+		o->oGetTextExtentW(txt, cb, &rx, &ry);
+		rx += 4;	rc.Xmin = -2.0;		rc.Ymin = 0.0;		rc.Xmax = rx;		rc.Ymax = ry;
+		si = sin(td->RotBL *0.01745329252);	csi = cos(td->RotBL *0.01745329252);
+		if(td->Align & TXA_HCENTER) {
+			rc.Xmin -= rx/2.0-1.0;		rc.Xmax -= rx/2.0-1.0;
+			}
+		else if(td->Align & TXA_HRIGHT) {
+			rc.Xmin -= rx-2.0;			rc.Xmax -= rx-2.0;
+			}
+		if(td->Align & TXA_VCENTER) {
+			rc.Ymin -= ry/2.0;			rc.Ymax -= ry/2.0;
+			}
+		else if(td->Align & TXA_VBOTTOM) {
+			rc.Ymin -= ry;				rc.Ymax -= ry;
+			}
+		SetMinMaxRect(&((BitMapWin*)o)->tr_rec, iround(rc.Xmin*csi + rc.Ymin*si)+x, iround(rc.Ymin*csi - rc.Xmin*si)+y,
+			iround(rc.Xmax*csi + rc.Ymin*si)+x, iround(rc.Ymin*csi - rc.Xmax*si)+y);
+		UpdateMinMaxRect(&((BitMapWin*)o)->tr_rec, iround(rc.Xmax*csi + rc.Ymax*si)+x, iround(rc.Ymax*csi - rc.Xmax*si)+y);
+		UpdateMinMaxRect(&((BitMapWin*)o)->tr_rec, iround(rc.Xmin*csi + rc.Ymax*si)+x, iround(rc.Ymax*csi - rc.Xmin*si)+y);
+		IncrementMinMaxRect(&((BitMapWin*)o)->tr_rec, (td->iSize>>1) +6);
+		((BitMapWin*)o)->tr_out = GetRectBitmap(&((BitMapWin*)o)->tr_rec, o);
+		((BitMapWin*)o)->tr_out->hres = ((BitMapWin*)o)->hres;
+		((BitMapWin*)o)->tr_out->vres = ((BitMapWin*)o)->vres;
+		memcpy(&ttd, td, sizeof(TextDEF));
+		ttd.ColTxt = td->ColTxt & 0x00ffffffL;	ttd.ColBg = td->ColBg & 0x00ffffffL;
+		((BitMapWin*)o)->tr_out->SetTextSpec(&ttd);
+		((BitMapWin*)o)->tr_out->oTextOutW(x-((BitMapWin*)o)->tr_rec.left, y-((BitMapWin*)o)->tr_rec.top,
+			txt, cb);
+		((BitMapWin*)o)->DoTransparency(td->ColTxt);
+		return true;
+		}
+	SelectObject(*dc, *hFont);					SetTextColor(*dc, (td->ColTxt)&0x00ffffffL);
+	SetBkColor(*dc, (td->ColBg)&0x00ffffffL);	SetBkMode(*dc, td->Mode ? TRANSPARENT : OPAQUE);
+	ix = iy = 0;								SetTextAlign(*dc, TA_LEFT | TA_TOP);
+	if((o->OC_type & 0xff) == OC_HIMETRIC) {
 		if(td->Style & TXS_SUB) {
 			if((td->Align & TXA_VCENTER) == TXA_VCENTER) iy -= o->un2iy(td->fSize*0.4);
 			else if((td->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= o->un2iy(td->fSize*0.2);
@@ -657,7 +696,7 @@ bool com_oTextOutW(int x, int y, w_char *txt, int cb, HFONT *hFont, HDC *dc,
 	dtrc.top = iy + y;				dtrc.right = dtrc.left+w;		dtrc.bottom = dtrc.top+h;
 	if(fabs(td->RotBL) >.01 || fabs(td->RotCHAR) >.01) {
 		SetGraphicsMode(*dc, GM_ADVANCED);
-		if(o->OC_type == OC_HIMETRIC) {
+		if((o->OC_type &0xff) == OC_HIMETRIC) {
 			if((td->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= (td->iSize<<1);
 			else if((td->Align & TXA_VCENTER) == TXA_VCENTER) iy -= (td->iSize);
 			xf.eM11 = (float)cos(td->RotBL *0.01745329252);
@@ -714,7 +753,7 @@ bool com_SetTextSpec(TextDEF *set, anyOutput *o, HFONT *hFont,	TextDEF *TxtSet,
 	LOGFONT FontRec;
 	HFONT newFont;
 
-	if(!set->iSize && set->fSize > 0.001f) set->iSize = o->un2iy(set->fSize);
+	if(!set->iSize && set->fSize > 0.001) set->iSize = o->un2iy(set->fSize);
 	if(!set->iSize) return false;
 	if(!*hFont || TxtSet->iSize != set->iSize || TxtSet->Style != set->Style ||
 		TxtSet->RotBL != set->RotBL || TxtSet->RotCHAR != set->RotCHAR ||
@@ -815,11 +854,10 @@ BitMapWin::BitMapWin(GraphObj *g, HWND hw):anyOutput()
 	Box1.Ymin = DeskRect.top;		Box1.Ymax = DeskRect.bottom;
 	scr = CreateCompatibleBitmap(dc, DeskRect.right, DeskRect.bottom);
 	memDC = CreateCompatibleDC(NULL);
-	SelectObject(memDC, scr);
-	ReleaseDC(hwndDesk, dc);
+	SelectObject(memDC, scr);		ReleaseDC(hwndDesk, dc);
 	hPen = CreatePen(PS_SOLID, 1, dLineCol = 0x00ffffffL);
 	hBrush = CreateSolidBrush(dFillCol =dBgCol = 0x00ffffffL);
-	dPattern = 0L;
+	dPattern = 0L;				minLW = 1;
 	if(memDC) {
 		oldPen = (HPEN)SelectObject(memDC, hPen);
 		oldBrush = (HBRUSH)SelectObject(memDC, hBrush);
@@ -828,7 +866,7 @@ BitMapWin::BitMapWin(GraphObj *g, HWND hw):anyOutput()
 		oldBrush = 0L;
 		oldPen = 0L;
 		}
-	hFont = 0L;
+	hFont = 0L;	OC_type = OC_BITMAP;
 }
 
 BitMapWin::BitMapWin(int w, int h, double hr, double vr)
@@ -837,7 +875,7 @@ BitMapWin::BitMapWin(int w, int h, double hr, double vr)
 	HWND hwndDesk;
 
 	memDC = 0L;		hgo = 0L;		go = 0L;
-	hres = hr;		vres = vr;
+	hres = hr;		vres = vr;		minLW = 1;
 	units = defs.cUnits;
 	DeskRect.right = w;				DeskRect.bottom = h;
 	DeskRect.left = DeskRect.top = 0;
@@ -854,7 +892,7 @@ BitMapWin::BitMapWin(int w, int h, double hr, double vr)
 		SelectObject(memDC, hPen);
 		SelectObject(memDC, hBrush);
 		}
-	hFont = 0L;
+	hFont = 0L;				OC_type = OC_BITMAP;
 }
 
 BitMapWin::BitMapWin(GraphObj *g):anyOutput()
@@ -864,8 +902,7 @@ BitMapWin::BitMapWin(GraphObj *g):anyOutput()
 
 	memDC = 0L;		hgo = 0L;		go = g;
 	dc = GetDC(hwndDesk = GetDesktopWindow());
-	hres = vres = 300.0;
-	units = defs.cUnits;
+	hres = vres = 300.0;	units = defs.cUnits;	minLW = 1;
 	DeskRect.right = un2ix(go->GetSize(SIZE_GRECT_RIGHT) - go->GetSize(SIZE_GRECT_LEFT));
 	DeskRect.bottom = un2iy(go->GetSize(SIZE_GRECT_BOTTOM) - go->GetSize(SIZE_GRECT_TOP));
 	DeskRect.top = DeskRect.left = 0;
@@ -882,19 +919,15 @@ BitMapWin::BitMapWin(GraphObj *g):anyOutput()
 		SelectObject(memDC, hPen);
 		SelectObject(memDC, hBrush);
 		}
-	hFont = 0L;
+	hFont = 0L;		OC_type = OC_BITMAP;
 }
 
 BitMapWin::~BitMapWin()
 {
-	Undo.KillDisp(this);
-	if(hgo) delete hgo;
-	if(hFont) DeleteObject(hFont);
-	if(scr) DeleteObject(scr);
-	SelectObject(memDC, oldPen);
-	SelectObject(memDC, oldBrush);
-	if(memDC) DeleteDC(memDC);
-	if(hPen) DeleteObject(hPen);
+	Undo.KillDisp(this);			if(hgo) delete hgo;
+	if(hFont) DeleteObject(hFont);		if(scr) DeleteObject(scr);
+	SelectObject(memDC, oldPen);		SelectObject(memDC, oldBrush);
+	if(memDC) DeleteDC(memDC);		if(hPen) DeleteObject(hPen);
 	if(hBrush) DeleteObject(hBrush);
 	hgo = 0L;	hFont = 0L;	scr = 0L;	
 	memDC = 0L;	hPen = 0L;	hBrush = 0L;
@@ -936,7 +969,7 @@ BitMapWin::SetFill(FillDEF *fill)
 		}
 	else {
 		if(hgo) delete hgo;
-		hgo = NULL;
+		hgo = 0L;
 		}
 	if(dFillCol != fill->color) {
 		newBrush = CreateSolidBrush(dFillCol = fill->color);
@@ -968,10 +1001,8 @@ BitMapWin::Erase(DWORD Color)
 			hOldBrush = (HBRUSH)SelectObject(memDC, hBGbrush);
 			hOldPen = (HPEN)SelectObject(memDC, hBGpen);
 			Rectangle(memDC, 0, 0, DeskRect.right, DeskRect.bottom);
-			SelectObject(memDC, hOldBrush);
-			SelectObject(memDC, hOldPen);
-			DeleteObject(hBGbrush);
-			DeleteObject(hBGpen);
+			SelectObject(memDC, hOldBrush);		SelectObject(memDC, hOldPen);
+			DeleteObject(hBGbrush);			DeleteObject(hBGpen);
 			return true;
 			}
 		if(hBGpen) DeleteObject(hBGpen);
@@ -1042,44 +1073,117 @@ bool
 BitMapWin::oCircle(int x1, int y1, int x2, int y2, char *nam)
 {
 	BOOL RetVal;
+	FillDEF tr_fill = {FILL_NONE, 0x0, 1.0, 0L, 0x0};
+	LineDEF tr_line = {0.0, 1.0, 0x0, 0x0};
+	bool bTrans = false;
+	HPEN newPen;
 	
-	RetVal = Ellipse(memDC, x1, y1, x2, y2);
-	if(RetVal && hgo) return hgo->oCircle(x1, y1, x2, y2);
-	else if(RetVal) return true;
-	return false;
+
+	if(dFillCol & 0xff000000) {
+		bTrans = true;
+		tr_fill.color = tr_fill.color2 = tr_line.color = (dFillCol & 0x00ffffffL);
+		SetMinMaxRect(&tr_rec, x1, y1, x2, y2);
+		IncrementMinMaxRect(&tr_rec, 6);
+		tr_out = GetRectBitmap(&tr_rec, this);
+		tr_out->hres = hres;			tr_out->vres = vres;
+		tr_out->SetLine(&tr_line);			tr_out->SetFill(&tr_fill);
+		RetVal = tr_out->oCircle(x1-tr_rec.left, y1-tr_rec.top, x2-tr_rec.left, y2-tr_rec.top, nam);
+		DoTransparency(dFillCol);
+		if(!(dLineCol & 0xff000000)) Arc(memDC, x1, y1,	x2, y2, 0, 0, 0, 0);
+		}
+	if(dLineCol & 0xff000000) {
+		if(!bTrans) {
+			newPen = CreatePen(PS_SOLID, 1, dFillCol);
+			SelectObject(memDC, newPen);			Ellipse(memDC, x1, y1, x2, y2);
+			if(hPen) {
+				SelectObject(memDC, hPen);			DeleteObject(newPen);
+				}
+			}
+		bTrans = true;
+		tr_line.color = (dLineCol & 0x00ffffffL);
+		tr_line.width = LineWidth;
+		tr_line.pattern = dPattern;
+		SetMinMaxRect(&tr_rec, x1, y1, x2, y2);
+		IncrementMinMaxRect(&tr_rec, 6 + un2ix(LineWidth*2.0));
+		tr_out = GetRectBitmap(&tr_rec, this);
+		tr_out->hres = hres;			tr_out->vres = vres;
+		tr_out->SetLine(&tr_line);
+		Arc(((BitMapWin*)tr_out)->memDC, x1-tr_rec.left, y1-tr_rec.top,
+			x2-tr_rec.left, y2-tr_rec.top, 0, 0, 0, 0);
+		DoTransparency(dLineCol);
+		}
+	if(!bTrans) {
+		RetVal = Ellipse(memDC, x1, y1, x2, y2);
+		if(RetVal && hgo) return hgo->oCircle(x1, y1, x2, y2);
+		else if(RetVal) return true;
+		}
+	return true;
 }
 
 bool
 BitMapWin::oPolyline(POINT * pts, int cp, char *nam)
 {
 	int i;
+	BOOL RetVal;
+	POINT *newpts;
+	LineDEF tr_line = {0.0, 1.0, 0x0, 0x0};
 
-	if(cp < 1) return false;
-	if (dPattern) {
-		for (i = 1; i < cp; i++) PatLine(pts[i-1], pts[i]);
-		return true;
+	if(!pts || cp < 1) return false;
+	if((dLineCol & 0xff000000) && (newpts = (POINT*)malloc(cp * sizeof(POINT)))) {
+		tr_line.color = (dLineCol & 0x00ffffffL);
+		tr_line.width = LineWidth;
+		tr_line.pattern = dPattern;
+		SetMinMaxRect(&tr_rec, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
+		for(i = 2; i < cp; i++) {
+			UpdateMinMaxRect(&tr_rec, pts[i].x, pts[i].y);
+			}
+		IncrementMinMaxRect(&tr_rec, 6 + un2ix(LineWidth*2.0));
+		tr_out = GetRectBitmap(&tr_rec, this);
+		tr_out->RLP.finc = RLP.finc;	tr_out->RLP.fp = RLP.fp;
+		tr_out->hres = hres;			tr_out->vres = vres;
+		for(i = 0; i < cp; i++) {
+			newpts[i].x = pts[i].x - tr_rec.left;
+			newpts[i].y = pts[i].y - tr_rec.top;
+			}
+		tr_out->SetLine(&tr_line);
+		RetVal = tr_out->oPolyline(newpts, cp, 0L);
+		RLP.finc = tr_out->RLP.finc;	RLP.fp = tr_out->RLP.fp;
+		DoTransparency(dLineCol);		free(newpts);
+		return (RetVal != 0);
 		}
 	else {
-		if(Polyline(memDC, pts, cp))return true;
-		else return false;
+		if (dPattern) {
+			for (i = 1; i < cp; i++) PatLine(pts[i-1], pts[i]);
+			return true;
+			}
+		else RetVal = Polyline(memDC, pts, cp);
 		}
+	return false;
 }
 
 bool
-BitMapWin::oRectangle(int x1, int y1, int x2, int y2, char *name)
+BitMapWin::oRectangle(int x1, int y1, int x2, int y2, char *nam)
 {
-	BOOL RetVal;
+	POINT pts[5];
 
-	RetVal = Rectangle(memDC, x1, y1, x2, y2);
-	if(RetVal && hgo) return hgo->oRectangle(x1, y1, x2, y2, 0L);
-	else if (RetVal) return true;
-	return false;
+	pts[0].x = pts[3].x = pts[4].x = x1;	pts[0].y = pts[1].y = pts[4].y = y1;
+	pts[1].x = pts[2].x = x2;		pts[2].y = pts[3].y = y2;
+	return oPolygon(pts, 5, nam);
 }
 
 bool
 BitMapWin::oSolidLine(POINT *p)
 {
-	if(Polyline(memDC, p, 2)) return true;
+	DWORD dPat;
+
+	if(dLineCol & 0xff000000L) {
+		dPat = dPattern;		dPattern = 0L;
+		oPolyline(p, 2, 0L);		dPattern = dPat;
+		return true;
+		}
+	else {
+		if(Polyline(memDC, p, 2)) return true;
+		}
 	return false;
 }
 
@@ -1098,12 +1202,93 @@ BitMapWin::oTextOutW(int x, int y, w_char *txt, int cb)
 bool
 BitMapWin::oPolygon(POINT *pts, int cp, char *nam)
 {
+	int i;
 	BOOL RetVal;
+	POINT *newpts;
+	FillDEF tr_fill = {FILL_NONE, 0x0, 1.0, 0L, 0x0};
+	LineDEF tr_line = {0.0, 1.0, 0x0, 0x0};
+	HPEN newPen;
 
-	RetVal = Polygon(memDC, pts, cp);
+	if(!pts || cp < 2) return false;
+	if((dFillCol & 0xff000000) && (newpts = (POINT*)malloc(cp * sizeof(POINT)))) {
+		tr_fill.color = tr_fill.color2 = tr_line.color = (dFillCol & 0x00ffffffL);
+		SetMinMaxRect(&tr_rec, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
+		for(i = 2; i < cp; i++) {
+			UpdateMinMaxRect(&tr_rec, pts[i].x, pts[i].y);
+			}
+		IncrementMinMaxRect(&tr_rec, 6);
+		tr_out = GetRectBitmap(&tr_rec, this);
+		tr_out->hres = hres;			tr_out->vres = vres;
+		for(i = 0; i < cp; i++) {
+			newpts[i].x = pts[i].x - tr_rec.left;
+			newpts[i].y = pts[i].y - tr_rec.top;
+			}
+		tr_out->SetLine(&tr_line);		tr_out->SetFill(&tr_fill);
+		RetVal = tr_out->oPolygon(newpts, cp, 0L);
+		DoTransparency(dFillCol);		free(newpts);
+		oPolyline(pts, cp, nam);
+		}
+	else if((dLineCol & 0xff000000) && (newpts = (POINT*)malloc(cp * sizeof(POINT)))) {
+		newPen = CreatePen(PS_SOLID, 1, dFillCol);
+		SelectObject(memDC, newPen);
+		RetVal = Polygon(memDC, pts, cp);
+		if(hPen) {
+			SelectObject(memDC, hPen);	oPolyline(pts, cp, nam);
+			DeleteObject(newPen);
+			}
+		}
+	else {
+		RetVal = Polygon(memDC, pts, cp);
+		}
 	if(RetVal && hgo) return hgo->oPolygon(pts, cp);
-	else if (RetVal) return true;
-	return false;
+	return RetVal != 0;
+}
+
+// The following code does alpha blending for transparent colors.
+//    The Windows AlphaBlend function requires msimg32.dll. To include
+//    this library add MSIMG32.LIB to the Project/Settings.../Link/Library_Moduls list.
+//    The code executed with USE_MSIMG32 undefined is very slow because of the GetPixel
+//    and SetPixel functions.
+//
+#define USE_MSIMG32
+void
+BitMapWin::DoTransparency(DWORD color)
+{
+#ifdef USE_MSIMG32
+	BLENDFUNCTION bf = {AC_SRC_OVER, 0, 127, 0};
+
+	bf.SourceConstantAlpha = (unsigned char)(255 - ((color >> 24) & 0xff));
+	AlphaBlend(memDC, tr_rec.left, tr_rec.top, tr_rec.right - tr_rec.left, tr_rec.bottom - tr_rec.top,
+		((BitMapWin*)tr_out)->memDC, 0, 0, tr_rec.right - tr_rec.left, tr_rec.bottom - tr_rec.top, bf);
+#else
+	int x1, y1, x2, y2, c, c1, c2;
+	DWORD col1, col2, col;
+	double f, f1;
+	
+	if(!tr_out) return;
+	f = ((color & 0xff000000L) >>24)/255.0;		f1 = 1.0 - f;
+	for(y1 = tr_rec.top, y2 = 0; y1 < tr_rec.bottom; y1++, y2++) {
+		for(x1 = tr_rec.left, x2 = 0; x1 < tr_rec.right; x1++, x2++) {
+			col1 = GetPixel(memDC, x1, y1);
+			col2 = GetPixel(((BitMapWin*)tr_out)->memDC, x2, y2);
+			if(col1 != col2) {
+				col = 0x0;
+				c1 = (col1 & 0x000000ffL);	c = c2 = (col2 & 0x000000ffL);
+				if(c1 != c2) c = (int)(c2 * f1 + c1 * f);
+				col |= (c < 256 ? c : 0xff);
+				c1 = ((col1 & 0x0000ff00L)>>8);	c = c2 = ((col2 & 0x0000ff00L)>>8);
+				if(c1 != c2) c = (int)(c2 * f1 + c1 * f);
+				col |= (c < 256 ? (c<<8) : 0x00ff00);
+				c1 = ((col1 & 0x00ff0000L)>>16);	c = c2 = ((col2 & 0x00ff0000L)>>16);
+				if(c1 != c2) c = (int)(c2 * f1 + c1 * f);
+				col |= (c < 256 ? (c<<16) : 0xff0000);
+				SetPixel(memDC, x1, y1, col);
+				}
+			}
+		}
+#endif	//USE_MSIMG32
+	DelBitmapClass(tr_out);			tr_out = 0L;
+	OC_type |= OC_TRANSPARENT;
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1111,7 +1296,7 @@ BitMapWin::oPolygon(POINT *pts, int cp, char *nam)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 OutputWin::OutputWin(GraphObj *g, HWND hw):BitMapWin(g, hw)
 {
-	hdc = 0L;
+	hdc = 0L;	minLW = 1;
 	if(g) {
 		if(!hw) CreateNewWindow(g);
 		else hWnd = hw;
@@ -1138,9 +1323,14 @@ OutputWin::ActualSize(RECT *rc)
 }
 
 void
-OutputWin::Caption(char *txt)
+OutputWin::Caption(char *txt, bool bModified)
 {
-	SetWindowText(hWnd, txt);
+	char txt1[200];
+	int cb;
+
+	cb = rlp_strcpy(txt1, 180, txt);
+	if(bModified)rlp_strcpy(txt1+cb, 20, " [modified]");
+	SetWindowText(hWnd, txt1);
 }
 
 const static unsigned char hand_bits[] =	{	//hand cursor bitmap
@@ -1531,7 +1721,7 @@ WinCopyWMF::WinCopyWMF(GraphObj *g, char *file_wmf, char *file_emf)
 	hPen = CreatePen(PS_SOLID, 1, dLineCol = 0x00ff0000L);
 	hBrush = CreateSolidBrush(dFillCol =dBgCol = 0x00ffffffL);
 	dPattern = 0L;		go = g;			hgo = 0L;		bott_y = 0;
-	dc = GetDC(hwndDesk = GetDesktopWindow());
+	dc = GetDC(hwndDesk = GetDesktopWindow());	minLW = 1;
 	hres = (double)GetDeviceCaps(dc, LOGPIXELSX);	vres = (double)GetDeviceCaps(dc, LOGPIXELSY);
 	ReleaseDC(hwndDesk, dc);	wmf_file = file_wmf;	emf_file = file_emf;
 }
@@ -1771,7 +1961,7 @@ PrintWin::PrintWin()
 
 	PrintDriver = PrintDevice = PrintPort = 0L;
 	hPen = 0L;		hBrush = 0L;			hFont = 0L;		hDC = 0L;
-	hgo = 0L;		units = defs.cUnits;	i = j = 0;
+	hgo = 0L;		units = defs.cUnits;		i = j = 0;		minLW = 1;
 	GetProfileString("windows", "device", "", TmpTxt, 4096);
 	while(TmpTxt[i] && TmpTxt[i] != ',') i++;
 	TmpTxt[i] = 0;
@@ -1927,6 +2117,16 @@ PrintWin::Eject()
 	return false;
 }
 
+bool
+PrintWin::CopyBitmap(int x, int y, anyOutput* sr, int sx, int sy,
+	int sw, int sh, bool invert)
+{
+	BitMapWin *src = (BitMapWin*)sr;
+
+	return(0 != BitBlt(hDC, x, y, sw, sh, src->memDC, sx, sy, 
+		invert ? DSTINVERT : SRCCOPY));
+}
+
 bool
 PrintWin::oCircle(int x1, int y1, int x2, int y2, char* nam)
 {
@@ -1955,14 +2155,13 @@ PrintWin::oPolyline(POINT * pts, int cp, char *nam)
 }
 
 bool
-PrintWin::oRectangle(int x1, int y1, int x2, int y2, char *name)
+PrintWin::oRectangle(int x1, int y1, int x2, int y2, char *nam)
 {
-	BOOL RetVal;
+	POINT pts[5];
 
-	RetVal = Rectangle(hDC, x1, y1, x2, y2);
-	if(RetVal && hgo) return hgo->oRectangle(x1, y1, x2, y2, 0L);
-	else if (RetVal) return true;
-	return false;
+	pts[0].x = pts[3].x = pts[4].x = x1;	pts[0].y = pts[1].y = pts[4].y = y1;
+	pts[1].x = pts[2].x = x2;		pts[2].y = pts[3].y = y2;
+	return oPolygon(pts, 5, nam);
 }
 
 bool
@@ -2082,6 +2281,7 @@ int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInstance,
 		rmquot(TmpTxt);
 		if(TmpTxt[0]) LoadFile= _strdup(TmpTxt);
 		}
+	ShellCmd = GetCommandLine();
 	hInstance = hInst;
 	wndclass.style = CS_BYTEALIGNWINDOW | CS_DBLCLKS;
 	wndclass.lpfnWndProc = WndProc;
@@ -2251,6 +2451,7 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 	MouseEvent mev;
 	HDC dc;
 	int cc;
+	RECT rec;
 
 	g = (GraphObj *) reflptr(GetWindowLong(hwnd, 0));
 	w = (OutputWin *) reflptr(GetWindowLong(hwnd, GWL_USERDATA));
@@ -2394,6 +2595,9 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				DestroyWindow(hwnd);
 				}
 			return 0;
+		case CM_NEWINST:
+			if(ShellCmd && ShellCmd[0])WinExec(ShellCmd, SW_SHOW);
+			return 0;
 		case CM_PASTE:
 			w->MouseCursor(MC_WAIT, true);
 			if(g->Id == GO_SPREADDATA) TestClipboard(g);
@@ -2423,7 +2627,7 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				SetClipboardData(cf_rlpxml, NULL);
 				}
 			else if(g->Id == GO_PAGE) {
-				SetClipboardData(CF_ENHMETAFILE, NULL);
+				if(!g->hasTransp()) SetClipboardData(CF_ENHMETAFILE, NULL);
 				SetClipboardData(CF_BITMAP, NULL);
 				if(CurrGraph) {
 					CopyGraph(CurrGraph, cf_rlpobj, w);		copy_obj = CurrGraph;
@@ -2431,12 +2635,12 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				}
 			else if (wParam == CM_CUT)return 0;
 			else if(CurrGraph && CurrGraph->Id == GO_GRAPH){
-				SetClipboardData(CF_ENHMETAFILE, NULL);
+				if(!g->hasTransp()) SetClipboardData(CF_ENHMETAFILE, NULL);
 				SetClipboardData(CF_BITMAP, NULL);
 				CopyGraph(CurrGraph, cf_rlpobj, w);			copy_obj = CurrGraph;
 				}
 			else if(g->Id == GO_GRAPH){
-				SetClipboardData(CF_ENHMETAFILE, NULL);
+				if(!g->hasTransp()) SetClipboardData(CF_ENHMETAFILE, NULL);
 				SetClipboardData(CF_BITMAP, NULL);
 				CopyGraph(g, cf_rlpobj, w);					copy_obj = g;
 				}
@@ -2499,11 +2703,11 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_DELGRAPH:
 			g->Command(CMD_DELGRAPH, 0L, w);
 			return 0;
-		case CM_SAVEDATA:
-			g->Command(CMD_SAVEDATA, 0L, w);
+		case CM_SAVE:
+			g->Command(CMD_SAVE, 0L, w);
 			return 0;
-		case CM_SAVEDATAAS:
-			g->Command(CMD_SAVEDATAAS, 0L, w);
+		case CM_SAVEAS:
+			g->Command(CMD_SAVEAS, 0L, w);
 			return 0;
 		case CM_REDRAW:
 			if(w->Erase(defs.Color(COL_BG))) g->DoPlot(w);
@@ -2516,9 +2720,6 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				}
 			else if(!CurrGO) InfoBox("No object selected!");
 			return 0;
-		case CM_SAVEGRAPHAS:
-			SaveGraphAs(g);
-			return 0;
 		case CM_EXPORT:
 			OpenExportName(g, 0L);
 			g->DoPlot(w);
@@ -2530,8 +2731,26 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				}
 			if(Printer && Printer->StartPage()) {
 				SetCursor(LoadCursor(0L, IDC_WAIT));
-				g->DoPlot(Printer);
-				Printer->EndPage();
+				rec.left = Printer->un2ix(g->GetSize(SIZE_GRECT_LEFT));
+				rec.right = Printer->un2ix(g->GetSize(SIZE_GRECT_RIGHT));
+				rec.top = Printer->un2iy(g->GetSize(SIZE_GRECT_TOP));
+				rec.bottom = Printer->un2iy(g->GetSize(SIZE_GRECT_BOTTOM));
+				if(g->hasTransp()) {
+					if((CopyBMP = new BitMapWin(rec.right-rec.left, rec.bottom-rec.top,
+						Printer->hres, Printer->vres)) && CopyBMP->StartPage()) {
+						CopyBMP->VPorg.fy = -Printer->co2fiy(g->GetSize(SIZE_GRECT_TOP));
+						CopyBMP->VPorg.fx = -Printer->co2fix(g->GetSize(SIZE_GRECT_LEFT));
+						g->DoPlot(CopyBMP);			CopyBMP->EndPage();
+						Printer->CopyBitmap(rec.left, rec.top, CopyBMP, 0, 0, CopyBMP->DeskRect.right,
+							CopyBMP->DeskRect.bottom, false);
+						delete CopyBMP;						CopyBMP = NULL;
+						Printer->EndPage();
+						}
+					}
+				else {
+					g->DoPlot(Printer);
+					Printer->EndPage();
+					}
 				w->Erase(defs.Color(COL_BG));
 				g->DoPlot(w);
 				SetCursor(LoadCursor(0L, IDC_ARROW));
@@ -2633,6 +2852,9 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_REPANOV:
 			if(g->data) rep_anova(g, g->data);
 			return 0;
+		case CM_REPBDANOV:
+			if(g->data) rep_bdanova(g, g->data);
+			return 0;
 		case CM_REPTWANR:
 			if(g->data) rep_twoway_anova(g, g->data);
 			return 0;
@@ -2827,7 +3049,7 @@ void *CreateDlgWnd(char *title, int x, int y, int width, int height, tag_DlgObj
 				(BoxRec.bottom- BoxRec.top)/2, BoxRec.right - BoxRec.left,
 				BoxRec.bottom - BoxRec.top, 0);
 			}
-		if(flags & 0x04)			SetTimer(hDlg, 1, 100, 0L);
+		if(flags & 0x08)			SetTimer(hDlg, 1, 100, 0L);
 		UpdateWindow(hDlg);			d->DoPlot(w);
 		ShowWindow(hDlg, SW_SHOW);
 		}
