@@ -1,4 +1,4 @@
-//rlp_math.cpp, Copyright (c) 2004-2007 R.Lackner
+//rlp_math.cpp, Copyright (c) 2004-2008 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -753,6 +753,49 @@ double weib_freq(double x, double shape, double scale)
 	return HUGE_VAL;
 }
 
+//---------------------------------------------------------------------------
+// The geometric distribution
+//---------------------------------------------------------------------------
+double geom_freq(double x, double p)
+{ 
+    if (p <= 0 || p > 1 || x < 0.0) return HUGE_VAL;
+	x = floor(x + 1.0e-16);
+	return pow(1.0 - p, x) * p;
+}
+
+double geom_dist(double x, double p)
+{
+	double sum, x1;
+
+	for(x1 = sum = 0.0; x1 <= x; sum += geom_freq(x1, p), x1 += 1.0);
+	return sum;
+}
+
+//---------------------------------------------------------------------------
+// The hypergeometric distribution
+//---------------------------------------------------------------------------
+double hyper_freq(double k, double n0, double m, double n1)
+{
+	double pr;
+
+	if(k < 0.0 || m < 0.0 || n1 < 0.0 || n1 > n0+m) return HUGE_VAL;
+	k = floor(k + 1.0e-16);		n0 = floor(n0 + 1.0e-16);
+	m = floor(m + 1.0e-16);		n1 = floor(n1 + 1.0e-16);
+
+	pr = gammln(m+1.0) - gammln(k+1.0) - gammln(m-k+1.0)
+		+ gammln(n0-m+1.0) - gammln(n1-k+1.0) - gammln(n0-m-n1+k+1.0)
+		- gammln(n0+1.0) + gammln(n1+1.0) + gammln(n0-n1+1.0);
+	return exp(pr);
+}
+
+double hyper_dist(double k, double n0, double m, double n1)
+{
+	double sum, x1;
+
+	for(x1 = sum = 0.0; x1 <= k; sum += hyper_freq(x1, n0, m, n1), x1 += 1.0);
+	return sum;
+}
+
 //---------------------------------------------------------------------------
 // The Cauchy (Lorentz) distribution
 //---------------------------------------------------------------------------
@@ -1302,7 +1345,9 @@ double d_spearman(double *sx, double *sy, int n, char *dest, DataObj *data, doub
 		|| !(y = (double*)memdup(sy, n*sizeof(double), 0)))return 0.0;
 	SortArray2(n, x, y);			crank(n, x, &sf);
 	SortArray2(n, y, x);			crank(n, y, &sg);
-	for(j = 0, res[0] = 0.0; j < n; j++) res[0] += ((tmp = (x[j]-y[j]))*tmp);
+	for(j = 0, res[0] = 0.0; j < n; j++){
+		res[0] += ((tmp = (x[j]-y[j]))*tmp);
+		}
 	en = n;						en3n = en*en*en -en;
 	aved = en3n/6.0 - (sf+sg)/12.0;
 	fac = (1.0-sf/en3n)*(1.0-sg/en3n);
@@ -2448,6 +2493,9 @@ void parse_datevalue(rlp_datetime *dt, double dv)
 			}
 		dt->month = i+1;				dt->dom = d;
 		}
+	else {
+		dt->aday = dt->year = dt->doy = dt->dom = dt->month = 0;
+		}
 	dv -= floor(dv);				dv *= 24.0;
 	dt->hours = (int)floor(dv);		dv -= floor(dv);
 	dv *= 60.0;						dt->minutes = (int)floor(dv); 
@@ -2581,24 +2629,3 @@ Triangle* Triangulate1(char *xr, char *yr, char *zr, DataObj *data)
 	if(rX) delete rX;	if(rY) delete rY;	if(rZ) delete rZ;
 	trl = tria->trl;	delete tria;		return trl;
 }
-
-Ribbon *SurfTria(GraphObj *parent, DataObj *data, char *xr, char *yr, char *zr)
-{
-	Triangle *trl, *trc, *trn;
-	int i, j, n, npl;
-	double tmp;
-	Plane3D **planes;
-
-	trl = Triangulate1(xr, zr, yr, data);
-	for(i = 0, trc = trl; trc; i++) trc = trc->next;
-	if((n = i) && (planes = (Plane3D**)malloc(n*sizeof(Plane3D*)))) 
-		for(i = npl = 0, trc = trl; trc && i < n; i++) {
-		for(j = 0; j < 4; j++) {	//swap y and z values;
-			tmp = trc->pt[j].fz;	trc->pt[j].fz = trc->pt[j].fy;	trc->pt[j].fy = tmp;
-			}
-		planes[npl++] = new Plane3D(0L, data, trc->pt, 4);
-		trn = trc->next;	delete trc;		trc = trn;
-		}
-	if(npl) return new Ribbon(parent, data, (GraphObj**)planes, npl);
-	return 0L;
-}
