@@ -1,4 +1,4 @@
-//RLPlot.h, Copyright (c) 2000-2007 R.Lackner
+//RLPlot.h, Copyright (c) 2000-2008 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -16,6 +16,8 @@
 //    along with RLPlot; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+#ifndef _RLPLOT_H
+#define _RLPLOT_H
 #define NUM_UNITS  3
 #define TMP_TXT_SIZE 4096
 
@@ -25,7 +27,7 @@
 #define Swap(a,b) {a^=b;b^=a;a^=b;}
 inline int iround(double a) {return a < 0.0 ?(int)(a-0.499) : (int)(a+0.499);}
 
-#define _PI		3.1415926535897932384626433832795028841971693993751
+#define _PI	3.1415926535897932384626433832795028841971693993751
 #define _SQRT2	1.4142135623730950488016887242096980785696718753769
 
 #ifdef _WINDOWS					//platform is windows
@@ -89,7 +91,7 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_ADDCHARW, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELE
 	CMD_BACKSP, CMD_CURRLEFT, CMD_CURRIGHT, CMD_CURRUP, CMD_CURRDOWN,
 	CMD_SHIFTLEFT, CMD_SHIFTRIGHT, CMD_SHIFTUP, CMD_SHIFTDOWN,
 	CMD_NEWGRAPH, CMD_DELGRAPH, CMD_DELOBJ, CMD_DROP_PLOT, CMD_DROP_GRAPH, CMD_OPEN,
-	CMD_SAVEDATAAS, CMD_ADDROWCOL, CMD_MOUSE_EVENT, CMD_REDRAW, CMD_DOPLOT,
+	CMD_SAVEAS, CMD_ADDROWCOL, CMD_MOUSE_EVENT, CMD_REDRAW, CMD_DOPLOT,
 	CMD_LBUP, CMD_ENDDIALOG, CMD_RADIOBUTT, CMD_ADDCHILD, CMD_BAR_TYPE,
 	CMD_BAR_FILL, CMD_SET_AXDEF, CMD_SET_DATAOBJ, CMD_SETTEXT, CMD_GETTEXT,
 	CMD_SETTEXTDEF, CMD_GETTEXTDEF, CMD_ADDPLOT, CMD_SYM_TYPE, CMD_SYMTEXT, 
@@ -118,8 +120,9 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_ADDCHARW, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELE
 	CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
 	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE, CMD_COPY, CMD_PASTE,
 	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, 
-	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVEDATA, CMD_GETMARK, CMD_PASTE_OBJ, CMD_COL_MOUSE,
-	CMD_MARKOBJ, CMD_SCALE, CMD_GETFILENAME, CMD_TEXTTHERE, CMD_HIDEMARK};
+	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVE, CMD_GETMARK, CMD_PASTE_OBJ, CMD_COL_MOUSE,
+	CMD_MARKOBJ, CMD_SCALE, CMD_GETFILENAME, CMD_TEXTTHERE, CMD_HIDEMARK, CMD_MENUHEIGHT,
+	CMD_RECALC, CMD_DRAWPG, CMD_UPDPG, CMD_MINMAX, CMD_STEP, CMD_SIGNAL_POL};
 enum {SYM_CIRCLE, SYM_CIRCLEF, SYM_RECT, SYM_RECTF, SYM_TRIAU, SYM_TRIAUF,
 	SYM_TRIAD, SYM_TRIADF, SYM_DIAMOND, SYM_DIAMONDF, SYM_PLUS, SYM_CROSS,
 	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_CIRCLEC, SYM_RECTC, SYM_TRIAUC, SYM_TRIAUL, SYM_TRIAUR,
@@ -140,9 +143,9 @@ enum {GO_UNKNOWN, GO_AXIS, GO_TICK, GO_GRIDLINE, GO_SYMBOL, GO_BUBBLE, GO_BAR,
 	GO_BOXPLOT, GO_DENSDISP, GO_STACKBAR, GO_STACKPG, GO_WATERFALL, GO_POLARPLOT,
 	GO_PIECHART, GO_RINGCHART, GO_GROUP, GO_STARCHART, GO_SCATT3D, GO_PLOT3D,
 	GO_RIBBON, GO_LIMITS, GO_FUNCTION, GO_FITFUNC, GO_FREQDIST, GO_GRID3D, GO_FUNC3D,
-	GO_XYSTAT, GO_FITFUNC3D, GO_NORMQUANT,
+	GO_XYSTAT, GO_FITFUNC3D, GO_NORMQUANT, GO_CONTOUR,
 	GO_GRAPH = 0x200, GO_PAGE, GO_SPREADDATA = 0x300, GO_DEFRW};
-enum {OC_UNKNOWN, OC_HIMETRIC};
+enum {OC_UNKNOWN, OC_BITMAP, OC_HIMETRIC, OC_TRANSPARENT = 0x100};
 enum {FILL_NONE, FILL_HLINES, FILL_VLINES, FILL_HVCROSS, FILL_DLINEU, FILL_DLINED,
 	FILL_DCROSS, FILL_STIPPLE1, FILL_STIPPLE2, FILL_STIPPLE3, FILL_STIPPLE4, 
 	FILL_STIPPLE5, FILL_ZIGZAG, FILL_COMBS, FILL_BRICKH, FILL_BRICKV, FILL_BRICKDU, 
@@ -367,14 +370,20 @@ class Triangle {
 public:
 	Triangle *next;
 	fPOINT3D pt[4];
+	int order[3];					//sort order
 
+	Triangle();
 	void SetRect();
 	bool TestVertex(double x, double y);
+	bool Sort();
+	void LinePoint(int i1, int i2, double z, lfPOINT *res);
+	bool IsoLine(double z, void *dest);
 
 private:
 	double cx, cy, r2;				//circumcircle
-	fRECT rc;						//bounding rectangle
+	fRECT rc;					//bounding rectangle
 	lfPOINT ld[3];					//line eqations
+	bool bSorted;					//vertices are sorted
 };
 
 class Triangulate {
@@ -395,24 +404,25 @@ private:
 
 class anyOutput{
 public:
-	int units;							//use units mm, inch ...
+	int units;						//use units mm, inch ...
+	int minLW;						//minimum line width in pix
 	int MrkMode;						//existing mark on screen
 	int OC_type;						//specify display, printer clipboard ...
 	void *MrkRect;						//pointer to e.g. the marked rectangle
-	fRECT Box1;							//User drawing rectangle
+	fRECT Box1;						//User drawing rectangle
 	lfPOINT Box1z;						// add 3D to Box1
 	RECT DeskRect;						//this is maximum size Rectangle
-	double ddx, ddy, ddz;				//convert to device coordinates
+	double ddx, ddy, ddz;					//convert to device coordinates
 	double hres, vres;					//resolution in dpi
 	double LineWidth;					//line width in units
-	int iLine;							//current width of line in pixels
+	int iLine;						//current width of line in pixels
 	DWORD dLineCol;						//current color of line;
 	DWORD dBgCol;						//color of background
 	DWORD dFillCol, dFillCol2;
 	DWORD dPattern;						//current line bit-pattern
 	TextDEF TxtSet;						//store current text settings here
 	RunLinePat RLP;						//continuous setings of pattern line
-	AxisDEF xAxis, yAxis, zAxis;		//axis and transform definitions
+	AxisDEF xAxis, yAxis, zAxis;				//axis and transform definitions
 	lfPOINT VPorg;						//zoom origin
 	double VPscale;						//zoom factor
 	int MenuHeight;						//ofset due to pull down menus
@@ -421,9 +431,9 @@ public:
 	fPOINT3D rotC;						//rotation center
 	bool hasHistMenu;					//File History List
 	int HistMenuSize;					//     -"-
-	lfPOINT light_source;				//angles for shading
-	double light_vec[3][3];				//     -"-
-	double disp_x, disp_y;				//displacement on page
+	lfPOINT light_source;					//angles for shading
+	double light_vec[3][3];					//     -"-
+	double disp_x, disp_y;					//displacement on page
 
 	anyOutput();
 	void SetRect(fRECT rec, int units, AxisDEF *x_ax, AxisDEF *y_ax);
@@ -456,7 +466,7 @@ public:
 	virtual bool SetLine(LineDEF *lDef){return false;};
 	bool GetLine(LineDEF *lDef);
 	virtual void Focus() {return;};
-	virtual void Caption(char *txt){return;};
+	virtual void Caption(char *txt, bool bModified){return;};
 	virtual void MouseCursor(int cid, bool force){return;};
 	virtual bool SetScroll(bool, int, int, int, int){return false;};
 	virtual bool SetFill(FillDEF *fill){return false;};
@@ -471,11 +481,13 @@ public:
 	virtual void ShowBitmap(int x, int y, anyOutput* src){return;};
 	bool ShowMark(void *rc, int Mode);
 	bool HideMark();
+	virtual void MouseCapture(bool bgrab){return;};
 	int CalcCursorPos(char *txt, POINT p, POINT *fit);
 	bool TextCursor(char *txt, POINT p, POINT *fit, int *pos, int disp);
 	bool PatLine(POINT p1, POINT p2);
 	virtual void ShowLine(POINT * pts, int cp, DWORD color) {return;};
 	virtual void ShowEllipse(POINT p1, POINT p2, DWORD color){return;}; 
+	virtual void ShowInvert(RECT *rec){return;};
 	virtual bool SetMenu(int type){return false;};
 	virtual void CheckMenu(int mid, bool check){return;};
 	virtual void FileHistory(){return;};
@@ -752,6 +764,7 @@ public:
 	virtual void * ObjThere(int x, int y);
 	virtual void Track(POINT *p, anyOutput *o);
 	virtual double DefSize(int select);
+	virtual bool hasTransp(){return false;};
 };
 
 class ssButton:public GraphObj {
@@ -933,6 +946,8 @@ public:
 	lfPOINT *Values;
 	lfPOINT min, max;
 	LineDEF LineDef;
+	FillDEF pgFill;
+	LineDEF pgFillLine;
 	long nPnt, nPntSet, cp;
 	DWORD BgColor;
 	POINT *pts;
@@ -943,7 +958,7 @@ public:
 	DataLine(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L, char *name=0L);
 	DataLine(GraphObj *par, DataObj *d, lfPOINT *val, long nval, char *name);
 	DataLine(int src);
-	~DataLine();
+	virtual ~DataLine();
 	bool SetColor(int select, DWORD col);
 	virtual void DoPlot(anyOutput *target);
 	virtual void DoMark(anyOutput *o, bool mark);
@@ -961,6 +976,7 @@ public:
 class DataPolygon:public DataLine{
 public:
 	DataPolygon(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L, char *name=0L);
+	DataPolygon(GraphObj *par, DataObj *d, lfPOINT *val, long nval, char *name = 0L);
 	DataPolygon(int src);
 	~DataPolygon();
 	void DoPlot(anyOutput *target);
@@ -968,10 +984,6 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	bool PropertyDlg();
 	bool FileIO(int rw);
-
-private:
-	FillDEF pgFill;
-	LineDEF pgFillLine;
 };
 
 class RegLine:public GraphObj{
@@ -1099,6 +1111,8 @@ public:
 	bool FileIO(int rw);
 
 private:
+	anyOutput *mo;
+	RECT mrc;
 	double size;
 	lfPOINT pos1, pos2;
 	POINT pts[5];
@@ -1243,10 +1257,10 @@ private:
 	FillDEF Fill;
 	double *PlaneVec;
 	POINT3D **ldata, ReqPoint;
-	int *nldata, nli, n_ipts, n_lines;
+	int *nldata, nli, n_ipts, n_lines, n_linept;
 	POINT *ipts;
 	lfPOINT xBounds, yBounds, zBounds;
-	bool bDrawDone, bReqPoint;
+	bool bDrawDone, bReqPoint, bSigPol;
 	GraphObj *co;
 	line_segment **lines;
 	long totalArea;
@@ -1526,6 +1540,8 @@ private:
 	FillDEF segFill;
 	POINT *pts;
 	bool bModified;
+	anyOutput *mo;
+	RECT mrc;
 };
 
 class polyline:public GraphObj {
@@ -1540,7 +1556,7 @@ public:
 
 	polyline(GraphObj *par, DataObj *d, lfPOINT *fpts, int cpts);
 	polyline(int src);
-	~polyline();
+	virtual ~polyline();
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	DWORD GetColor(int select);
@@ -1595,7 +1611,7 @@ public:
 
 	rectangle(GraphObj *par, DataObj *d, lfPOINT *p1, lfPOINT *p2);
 	rectangle(int src);
-	~rectangle();
+	virtual ~rectangle();
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	DWORD GetColor(int select){return Line.color;};
@@ -1698,6 +1714,7 @@ public:
 	TextValue *x_tv, *y_tv;			//TextValue object for ordinal axes
 
 	Plot(GraphObj *par, DataObj *d);
+	~Plot(){return;};
 	virtual double GetSize(int select);
 	virtual bool SetSize(int select, double value){return false;};
 	virtual DWORD GetColor(int select);
@@ -1705,6 +1722,8 @@ public:
 	virtual void DoPlot(anyOutput *o){return;};
 	virtual bool Command(int cmd, void *tmpl, anyOutput *o){return false;};
 	virtual bool PropertyDlg(){return false;};
+	virtual void RegGO(void *n){return;};
+	virtual bool FileIO(int rw) {return false;};
 
 	void CheckBounds(double x, double y);
 	bool UseAxis(int idx);
@@ -1730,7 +1749,7 @@ public:
 	PlotScatt(GraphObj *par, DataObj *d, int cBars, Bar **bars, ErrorBar **errs);
 	PlotScatt(GraphObj *par, DataObj *d, int nPts, Symbol **sym, DataLine *lin);
 	PlotScatt(int src);
-	~PlotScatt();
+	virtual ~PlotScatt();
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	bool SetColor(int select, DWORD col);
@@ -1934,7 +1953,7 @@ public:
 
 	StackBar(GraphObj *par, DataObj *d);
 	StackBar(int src);
-	~StackBar();
+	virtual ~StackBar();
 	bool SetSize(int select, double value);
 	void DoPlot(anyOutput *o);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
@@ -1979,7 +1998,7 @@ class PieChart:public Plot {
 public:
 	PieChart(GraphObj *par, DataObj *d);
 	PieChart(int src);
-	~PieChart();
+	virtual ~PieChart();
 	bool SetSize(int select, double value);
 	void DoPlot(anyOutput *o);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
@@ -2009,7 +2028,7 @@ public:
 
 	GoGroup(GraphObj *par, DataObj *d);
 	GoGroup(int src);
-	~GoGroup();
+	virtual ~GoGroup();
 	double GetSize(int select);
 	void DoPlot(anyOutput *o);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
@@ -2198,7 +2217,7 @@ public:
 
 	GridLine(GraphObj *par, DataObj *d, int type, DWORD df);
 	GridLine(int src);
-	~GridLine();
+	virtual ~GridLine();
 	virtual void DoPlot(anyOutput *o);
 	virtual void DoMark(anyOutput *o, bool mark);
 	virtual bool Command(int cmd, void *tmpl, anyOutput *o);
@@ -2239,20 +2258,29 @@ public:
 	bool PropertyDlg();
 	void RegGO(void *n);
 	bool FileIO(int rw);
+	void Track(POINT *p, anyOutput *o);
 
 	void DoPlot(double six, double csx, anyOutput *o);
+	bool ProcSeg();
 	
 private:
 	double value, size, fix, fiy, fiz, lsi, lcsi, angle, lbx, lby;
-	bool bModified;
+	int n_seg, s_seg;
+	double *seg;
+	bool bModified, bValidTick;
+	long numPG;
 	anyOutput *mo;
 	RECT mrc;
 	int gl_type;
 	GridLine *Grid;
 	Label *label;
+	DataPolygon **Polygons;
 	DWORD flags;
 	POINT pts[2];
 	line_segment *ls;
+
+	bool CmpPoints(double x1, double y1, double x2, double y2);
+	bool StoreSeg(lfPOINT *line);
 };
 
 class Axis:public GraphObj{
@@ -2260,6 +2288,8 @@ public:
 	AxisDEF *axis;
 	LineDEF axline;
 	TextValue *atv;
+	long NumTicks;
+	Tick **Ticks;
 
 	Axis(GraphObj *par, DataObj *d, AxisDEF *ax, DWORD flags);
 	Axis(int src);
@@ -2274,25 +2304,27 @@ public:
 	bool PropertyDlg();
 	void RegGO(void *n);
 	bool FileIO(int rw);
+	void *ObjThere(int x, int y);
+	void Track(POINT *p, anyOutput *o);
 
 	AxisDEF *GetAxis() {return axis;};
 	bool GetValuePos(double val, double *fix, double *fiy, double *fiz, anyOutput *o);
 	void TickFile(char *name);
 	void BreakSymbol(POINT3D *p1, double dsi, double dcsi, bool connect, anyOutput *o);
 	void DrawBreaks(anyOutput *o);
+	void CreateTicks();
+	DWORD GradColor(double value);
 
 private:
 	double sizAxLine, sizAxTick, sizAxTickLabel;
 	double si, csi;
 	double brksymsize, brkgap, tick_angle;
-	int brksym, nl_segs, gl_type, tick_type;
+	int brksym, nl_segs, gl_type, tick_type, grad_type;
 	bool bModified;
-	DWORD colAxis;
+	DWORD colAxis, gCol_0, gCol_1, gCol_2, gTrans;
 	LineDEF GridLine;
-	Tick **Ticks;
 	GraphObj *axisLabel;
-	long NumTicks;
-	POINT pts[2];
+	POINT pts[2], gradient_box[5];
 	POINT3D pts3D[2];
 	fPOINT3D flim[2];
 	lfPOINT lbdist, tlbdist;
@@ -2305,11 +2337,40 @@ private:
 
 	void SetTick(long idx, double val, DWORD flags, char *txt);
 	void mkTimeAxis();
-	void CreateTicks();
 	void ManuTicks(double sa, double st, int n, DWORD flags);
 	void UpdateTicks();
 	bool ssTicks();
+	void GradientBar(anyOutput *o);
+};
 
+class ContourPlot:public Plot {
+public:
+	ContourPlot(GraphObj *par, DataObj *d);
+	ContourPlot(int src);
+	~ContourPlot();
+	bool SetSize(int select, double value);
+	bool SetColor(int select, DWORD col);
+	void DoPlot(anyOutput *o);
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	void RegGO(void *n);
+	bool FileIO(int rw);
+
+private:
+	char *ssRefX, *ssRefY, *ssRefZ;
+	Symbol **Symbols;
+	Label **Labels;
+	long nval, nSym, nLab;
+	DWORD flags;
+	double sr_zval;
+	fPOINT3D *val;
+	AxisDEF z_axis;
+	Axis *zAxis;
+
+	bool LoadData(char *xref, char *yref, char *zref);
+	bool DoTriangulate();
+	bool DoAxis(anyOutput *o);
+	void DoSymbols(Triangle *trl);
 };
 
 class Plot3D:public Plot{
@@ -2327,7 +2388,7 @@ public:
 
 	Plot3D(GraphObj *par, DataObj *d, DWORD flags);
 	Plot3D(int src);
-	~Plot3D();
+	virtual ~Plot3D();
 	double GetSize(int select);
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *o);
@@ -2355,6 +2416,7 @@ private:
 	obj_desc **dispObs;
 
 	bool AddPlot(int family);
+	int cmp_obj_desc(obj_desc *obj1, obj_desc *obj2);
 };
 
 class Chart25D:public Plot3D {
@@ -2444,7 +2506,7 @@ public:
 
 	Graph(GraphObj *par, DataObj *d, anyOutput *o, int style);
 	Graph(int src);
-	~Graph();
+	virtual ~Graph();
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	DWORD GetColor(int select);
@@ -2455,6 +2517,7 @@ public:
 	void RegGO(void *n);
 	virtual bool FileIO(int rw);
 	virtual double DefSize(int select);
+	bool hasTransp();
 
 private:
 	int NumAxes, AxisTempl, tickstyle, zoom_level;
@@ -2534,7 +2597,7 @@ private:
 
 class Default{
 public:
-	int dUnits, cUnits;
+	int dUnits, cUnits, iMenuHeight;
 	char DecPoint[2], ColSep[2];
 	char *svgAttr, *svgScript, *currPath, *IniFile;
 	char *File1, *File2, *File3, *File4, *File5, *File6;
@@ -2558,6 +2621,9 @@ public:
 	FillDEF *pgFillDEF(FillDEF *fd);
 	double rrectRad(double rad);
 	void FileHistory(char *path);
+	void Idle(int cmd);
+	void UpdRect(anyOutput *o, int x1, int y1, int x2, int y2);
+	void UpdAdd(anyOutput * o, int x, int y);
 
 private:
 	LineDEF Line_0, Line_1, Line_2, *pl, *pgl;
@@ -2565,7 +2631,9 @@ private:
 	LineDEF FillLine_0, FillLine_1, FillLine_2, *pg_fl;
 	LineDEF OutLine_0, OutLine_1, OutLine_2;
 	double *rrect_rad;
-	anyOutput *cdisp;
+	anyOutput *cdisp, *out_upd;
+	RECT rec_upd;
+	bool can_upd;
 	DWORD axis_color;
 };
 
@@ -2813,9 +2881,9 @@ bool IsInPolygon(POINT *p, POINT *pts, int cp);
 bool OverlapRect(RECT *rc1, RECT *rc2);
 void AddToPolygon(long *cp, POINT *pts, POINT *np);
 void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth=0);
+void ClipBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, POINT *clp1, POINT *clp2);
 int mkCurve(lfPOINT *src, int n1, lfPOINT **dst, bool bClosed);
 POINT *MakeArc(int ix, int iy, int r, int qad, long *npts);
-void InvertPolygon(POINT*, int, LineDEF*, FillDEF*, RECT*, anyOutput*, bool);
 void InvertLine(POINT*, int, LineDEF*, RECT*, anyOutput*, bool);
 unsigned int ColDiff(DWORD col1, DWORD col2);
 DWORD IpolCol(DWORD color1, DWORD color2, double fact);
@@ -2911,6 +2979,10 @@ double f_dist(double f, double df1, double df2);
 double f_freq(double f, double df1, double df2);
 double weib_dist(double x, double shape, double scale);
 double weib_freq(double x, double shape, double scale);
+double geom_freq(double x, double p);
+double geom_dist(double x, double p);
+double hyper_freq(double k, double n0, double m, double n1);
+double hyper_dist(double k, double n0, double m, double n1);
 double cauch_dist(double x, double loc, double scale);
 double cauch_freq(double x, double loc, double scale);
 double logis_dist(double x, double loc, double scale);
@@ -2956,10 +3028,10 @@ char *value_date(double dv, char *fmt);
 double now_today();
 void split_date(double dv, int *y, int *mo, int *dom, int *dow, int *doy, int *h, int *m, double *s);
 Triangle* Triangulate1(char *xr, char *yr, char *zr, DataObj *data);
-Ribbon *SurfTria(GraphObj *parent, DataObj *data, char *text1, char *text2, char *text3);
 
 //prototypes reports.cpp
 void rep_anova(GraphObj *parent, DataObj *data);
+void rep_bdanova(GraphObj *parent, DataObj *data);
 void rep_twanova(GraphObj *parent, DataObj *data);
 void rep_fmanova(GraphObj *parent, DataObj *data);
 void rep_twoway_anova(GraphObj *parent, DataObj *data);
@@ -2970,4 +3042,4 @@ void rep_robustline(GraphObj *parent, DataObj *data);
 void rep_twowaytable(GraphObj *parent, DataObj *data);
 void rep_compmeans(GraphObj *parent, DataObj *data);
 void rep_correl(GraphObj *parent, DataObj *data, int style);
-
+#endif //_RLPLOT_H
