@@ -1,4 +1,4 @@
-//spreadwin.cpp, (c)2000-2006 by R. Lackner
+//spreadwin.cpp, (c)2000-2008 by R. Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -43,6 +43,7 @@ extern UndoObj Undo;
 
 static ReadCache *Cache = 0L;
 static TextDEF ssText;
+static char *szRlpData = "RLPlot data";
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Get item from *.csv file
@@ -166,7 +167,7 @@ SpreadWin::SpreadWin(GraphObj *par, DataObj *Data):GraphObj(par, Data)
 		ssText.ColTxt = 0x00000000L;				ssText.text = 0L;
 		ssText.Font = FONT_HELVETICA;				w->SetTextSpec(&ssText);
 		w->SetMenu(MENU_SPREAD);					w->FileHistory();
-		w->Erase(0x00e8e8e8L);						w->Caption("RLPlot data");
+		w->Erase(0x00e8e8e8L);						w->Caption(szRlpData, false);
 		d->ri->SetDefWidth(w->un2ix(defs.GetSize(SIZE_CELLWIDTH)));
 		d->ri->SetHeight(w->un2iy(defs.GetSize(SIZE_CELLTEXT)/defs.ss_txt) + 2);
 		d->ri->SetFirstWidth(32);
@@ -216,6 +217,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 	char *Name;
 	Graph *g2;
 	int i, j, k;
+	bool bRet;
 	MouseEvent *mev;
 	POINT p1, p2;
 
@@ -252,10 +254,13 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 				if(Undo.isEmpty(0L)) return true;
 				i = YesNoCancelBox("The spreadsheet or a graph has been modified!\n\nDo you want to save it now?");
 				if(i == 2) return false;
-				else if(i == 1) return Command(CMD_SAVEDATAAS, tmpl, o);
+				else if(i == 1) return Command(CMD_SAVEAS, tmpl, o);
 				}
 			return true;
 		case CMD_MRK_DIRTY:
+			if(!is_modified) {
+				o->Caption(filename && filename[0]?filename : szRlpData, true);
+				}
 			return is_modified = true;
 		case CMD_WRITE_GRAPHS:
 			if (g && NumGraphs) WriteGraphXML((unsigned char**)tmpl, (long*)o);
@@ -310,19 +315,21 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 				}
 			if(g && j < i) g[j] = 0L;			NumGraphs = j;
 			return true;
-		case CMD_SAVEDATA:
+		case CMD_SAVE:
 			if(o) o->MouseCursor(MC_WAIT, false);
 			if(d->WriteData(0L)) {
+				o->Caption(filename && filename[0]?filename : szRlpData, false);
 				is_modified=false;
 				if(o) o->MouseCursor(MC_ARROW, false);
 				return true;
 				}
 			if(o) o->MouseCursor(MC_ARROW, true);
-		case CMD_SAVEDATAAS:
+		case CMD_SAVEAS:
 			is_modified=false;
 			if((Name = SaveDataAsName(filename)) && Name[0]){
 				if(o) o->FileHistory();
 				if(Name && d->WriteData(Name)) {
+					o->Caption(filename && filename[0]?filename : szRlpData, false);
 					if(filename) free(filename);
 					filename = (char*)memdup(Name, (int)strlen(Name)+1, 0);
 					}
@@ -336,6 +343,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			else if(d->ReadData((char*)tmpl, 0L, FF_UNKNOWN)){
 				if(filename) free(filename);
 				filename = (char*)memdup(tmpl, (int)strlen((char*)tmpl)+1, 0);
+				o->Caption(filename && filename[0]?filename : szRlpData, is_modified = false);
 				return Command(CMD_SETSCROLL, 0L, w);
 				}
 			else ErrorBox("The selected file is not valid\nor not accessible!\n");
@@ -428,7 +436,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 				else if(mev->y < (o->MenuHeight + d->ri->GetHeight(-1)) && cButtons) {
 					if(!(d->mpos2dpos(&p1, &p2, false)))return false;
 					p2.x -= ssOrg.x;		p2.y -= ssOrg.y;
-					if (p2.x < 0 || p2.x >= (d->c_disp-1)) return false;
+					if (p2.x < 0 || p2.x >= d->c_disp) return false;
 					if(cButtons[p2.x]) cButtons[p2.x]->Command(cmd, tmpl, o);
 					}
 				else if(o->MrkMode == MRK_SSB_DRAW) o->HideMark();
@@ -440,22 +448,28 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 		case CMD_COPY_TSV:		case CMD_COPY_XML:		case CMD_QUERY_COPY:
 		case CMD_TAB:			case CMD_SHTAB:			case CMD_SHIFTDOWN:
 		case CMD_CURRLEFT:		case CMD_CURRIGHT:		case CMD_CURRUP:
-		case CMD_CURRDOWN:		case CMD_SHIFTRIGHT:	case CMD_POS_FIRST:
+		case CMD_CURRDOWN:		case CMD_SHIFTRIGHT:		case CMD_POS_FIRST:
 		case CMD_POS_LAST:		case CMD_SHIFTLEFT:		case CMD_DELETE:
 		case CMD_TOOLMODE:		case CMD_FILLRANGE:		case CMD_CUT:
 		case CMD_PASTE_XML:		case CMD_DELROW:		case CMD_INSROW:
 		case CMD_INSCOL:		case CMD_DELCOL:		case CMD_UNDO:
 		case CMD_SHPGUP:		case CMD_SHPGDOWN:		case CMD_HIDEMARK:
-			bDoColWidth = false;
-			return d->Command(cmd, tmpl, o);
+			bDoColWidth = false;	bRet = d->Command(cmd, tmpl, o);
+			if(cmd == CMD_UNDO && is_modified && Undo.isEmpty(0L)) {
+				w->Caption(filename && filename[0]?filename : szRlpData, is_modified = false);
+				}
+			return bRet;
+		case CMD_MENUHEIGHT:
+			if(w) w->MenuHeight = defs.iMenuHeight;
+			bDoColWidth = true;
 		case CMD_REDRAW:
-			Undo.SetDisp(w);						d->Command(cmd, tmpl, o);
+			Undo.SetDisp(w);					d->Command(cmd, tmpl, o);
 			return true;
 		case CMD_MOUSECURSOR:
 			if(o)o->MouseCursor(MC_ARROW, false);	
 			return true;
 		case CMD_SETSCROLL:
-			HideTextCursor();						if(!(o->ActualSize(&currRC)))return false;
+			HideTextCursor();					if(!(o->ActualSize(&currRC)))return false;
 			k = (currRC.bottom-currRC.top)/d->ri->GetHeight(-1);
 			d->GetSize(&i, &j);			o->MouseCursor(MC_WAIT, true);
 			o->SetScroll(true, 0, j, k, ssOrg.y);	k = (currRC.right-currRC.left)/d->ri->GetWidth(-1);
@@ -926,6 +940,7 @@ SpreadData::Init(int nRows, int nCols)
 
 	cRows = nRows;	cCols = nCols;				currpos.x = currpos.y = 0;
 	new_mark = bCopyCut = false;				if(w && m_range) HideMark(true);
+	bUpdate = true;
 	while(ri->Type()) {
 		o_ri = ri;		ri = ri->Next();		delete(o_ri);
 		}
@@ -1254,6 +1269,7 @@ SpreadData::ReadData(char *FileName, unsigned char *buffer, int type)
 		}while (ItemText[0] || !Cache->IsEOF());		//eof
 	Cache->Close();		delete Cache;		Cache = 0L;
 	Disp->ssOrg.x = Disp->ssOrg.y = 0;
+	bUpdate = true;
 	return true;
 
 ReadError:
@@ -1378,6 +1394,7 @@ SpreadData::DeleteCols()
 		free(cs);			delete ar;			return false;
 		}
 	ar->NextCol(&c1);		cn = c1;			nc = 0;
+	Disp->Command(CMD_MRK_DIRTY, 0L, 0L);
 	do {
 		for(c0 = c1 = c2 = cn; ar->NextCol(&c2); ) {
 			if(c2 > c1+1 || c2 < c0) break;
@@ -1466,7 +1483,7 @@ SpreadData::InsertCols()
 		dc = c1-c0+1;		cn = c2;
 		cs[nc].fx = c0;		cs[nc].fy = c1;		nc++;
 		}while(c2!=c1);
-	SortFpArray(nc, cs);	nc--;
+	SortFpArray(nc, cs);	nc--;	Disp->Command(CMD_MRK_DIRTY, 0L, w);
 	do {
 		c0 = (int)cs[nc].fx;	c1 = (int)cs[nc].fy;	dc = c1-c0+1;
 		if(AddCols(cCols + dc)) for(i = 0; i < cRows; i++) {
@@ -1530,6 +1547,7 @@ SpreadData::DeleteRows()
 		free(rs);			delete ar;			return false;
 		}
 	ar->NextRow(&r1);		rn = r1;			nr = 0;
+	Disp->Command(CMD_MRK_DIRTY, 0L, w);
 	do {
 		for(r0 = r1 = r2 = rn;ar->NextRow(&r2); ) {
 			if(r2 > r1+1 || r2 < r0) break;
@@ -1610,6 +1628,7 @@ SpreadData::InsertRows()
 		free(rs);			delete ar;			return false;
 		}
 	ar->NextRow(&r1);		rn = r1;			nr = 0;
+	Disp->Command(CMD_MRK_DIRTY, 0L, w);
 	do {
 		for(r0 = r1 = r2 = rn;ar->NextRow(&r2); ) {
 			if(r2 > r1+1 || r2 < r0) break;
@@ -1671,6 +1690,7 @@ SpreadData::DoPlot(anyOutput *o)
 	RECT rc;
 	int i, j, r, c;
 	AccRange *ar;
+	double v;
 
 	if(!w || !Disp) return;
 	w->Erase(0x00e8e8e8L);					if(!(w->ActualSize(&rc)))return;
@@ -1683,6 +1703,12 @@ SpreadData::DoPlot(anyOutput *o)
 	Disp->ShowGrid(CellWidth, CellHeight, FirstWidth, &currpos);
 	rc.top = w->MenuHeight;					rc.bottom = rc.top + CellHeight;
 	LockData(false, false);
+	if(bUpdate) {
+		for(i = cRows-1; i >= 0; i--) {
+			for(j = cCols-1; j >= 0; j--) etRows[i][j]->GetValue(&v);
+			}
+		bUpdate = false;
+		}
 	for(i = 0; i <= (cRows - Disp->ssOrg.y) && i <= r_disp; i++) {
 		rc.left = ri->GetFirstWidth();		rc.right = rc.left + ri->GetWidth(Disp->ssOrg.x);
 		rc.top += CellHeight;				rc.bottom += CellHeight;
@@ -1696,20 +1722,9 @@ SpreadData::DoPlot(anyOutput *o)
 			rc.right += ri->GetWidth(Disp->ssOrg.x+j+1);
 			}
 		}
-	if(bUpdate) {
-		for(i = 0; i <= (cRows - Disp->ssOrg.y) && i <= r_disp; i++) {
-			for(j = 0; j <= (cCols - Disp->ssOrg.x) && j <= c_disp; j++) {
-				r = i + Disp->ssOrg.y;	c = j + Disp->ssOrg.x; 
-				if(r < cRows && r >= 0 && c < cCols && c >=0 && etRows[r][c]){
-					etRows[r][c]->Redraw(w, false);
-					}
-				}
-			}
-		}
 	if(CurrText && CurrText->row >= Disp->ssOrg.y && CurrText->row < (Disp->ssOrg.y + r_disp)
 			&& CurrText->col >= Disp->ssOrg.x && CurrText->col < (Disp->ssOrg.x +c_disp))
 			CurrText->Update(1, w, 0L);
-	bUpdate = false;
 	if(m_range && (ar = new AccRange(m_range)) && ar->GetFirst(&c, &r)) {
 		for( ; ar->GetNext(&c, &r); ) {
 			if(r >= Disp->ssOrg.y && r < (r_disp + Disp->ssOrg.y) && c >= Disp->ssOrg.x 
@@ -1736,6 +1751,7 @@ SpreadData::DelRange()
 	RECT rec;
 
 	if(m_range && (ar = new AccRange(m_range)) && ar->GetFirst(&c, &r)) {
+		Disp->Command(CMD_MRK_DIRTY, 0L, w);
 		ar->BoundRec(&rec);
 		Undo.DataObject(Disp, w, this, &rec, 0L);
 		while(ar->GetNext(&c, &r)) {
@@ -1759,7 +1775,7 @@ SpreadData::PasteRange(int cmd, char *txt)
 	RECT mrk_range;
 
 	if(new_mark && m_range && (cr = new AccRange(m_range)) && cr->GetFirst(&c, &r)) {
-		cr->BoundRec(&mrk_range);
+		cr->BoundRec(&mrk_range);	Disp->Command(CMD_MRK_DIRTY, 0L, w);
 		for(i = 0 ; cr->GetNext(&c, &r); i++) if(c >= 0 && c < cCols && r >= 0 && r < cRows){
 			currpos.x = c;	currpos.y = r;
 			switch(cmd){
@@ -1984,6 +2000,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 			}
 		break;
 	case CMD_PASTE_TSV:		case CMD_PASTE_XML:	case CMD_PASTE_CSV:	case CMD_PASTE_SSV:
+		bUpdate = true;		Disp->Command(CMD_MRK_DIRTY, 0L, 0L);
 		if(PasteRange(cmd, (char*)tmpl)) 
 			return Disp->Command(CMD_SETSCROLL, 0L, w);
 		return false;
@@ -2203,6 +2220,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 				CurrText->Update(1, w, 0L);
 				}
 			}
+		
 		return true;
 	case CMD_DELETE:
 		if(m_range) DelRange();
@@ -2234,6 +2252,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		return MemList((unsigned char**)tmpl, FF_SYLK);
 	case CMD_COPY_XML:
 		return MemList((unsigned char**)tmpl, FF_XML);
+	case CMD_MENUHEIGHT:
 	case CMD_DOPLOT:	case CMD_REDRAW:
 		if(CurrText) CurrText->Update(2, 0L, 0L);
 		if(etRows && etRows[currpos.y] && currpos.y < cRows && currpos.x < cCols) 
@@ -2242,7 +2261,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		break;
 	case CMD_FILLRANGE:
 		Undo.SetDisp(w);
-		FillSsRange(this, &m_range, Disp);
+		if(FillSsRange(this, &m_range, Disp)) Disp->Command(CMD_MRK_DIRTY, 0L, o);
 		DoPlot(o);
 		Undo.SetDisp(w);
 		break;
@@ -2398,7 +2417,7 @@ SpreadData::ReadXML(char *file, unsigned char *buffer, int type, DWORD undo_flag
 					if(row >= cRows)AddRows(row+1);
 					if(col >= cCols)AddCols(col+1);
 					if(i && etRows[row] && etRows[row][col]) {
-						if(TmpTxt[0] == '=') {
+						if(TmpTxt[0] == '=' && (currpos.x || currpos.y || mov.x || mov.y)) {
 							MoveFormula(this, TmpTxt, TmpTxt, TMP_TXT_SIZE, currpos.x-mov.x, currpos.y-mov.y, -1, -1);
 							}
 						etRows[row][col]->SetText(TmpTxt);
