@@ -1,4 +1,4 @@
-//use_gui.cpp, Copyright 2000-2007 R.Lackner
+//use_gui.cpp, Copyright 2000-2008 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -71,10 +71,10 @@ ssButton::DoPlot(anyOutput *o)
 		}
 	o->SetLine(&Line);				o->SetFill(&Fill);
 	if(bLBdown){
-		o->oRectangle(rDims.left, rDims.top, rDims.right, rDims.bottom);
+		o->oRectangle(rDims.left, rDims.top, rDims.right-1, rDims.bottom-1);
 		}
 	else {
-		o->oRectangle(rDims.left, rDims.top, rDims.right-1, rDims.bottom-1);
+		o->oRectangle(rDims.left, rDims.top, rDims.right-2, rDims.bottom-2);
 		Line.color = 0x00000000L;
 		o->SetLine(&Line);
 		pts[0].x = rDims.left;					pts[0].y = pts[1].y = rDims.bottom-1;
@@ -295,7 +295,11 @@ dragHandle::DoPlot(anyOutput *o)
 			}
 		else {
 			IncrementMinMaxRect(&rDims, -1);
+#ifdef _WINDOWS
 			o->UpdateRect(&rDims, true);
+#else
+			o->ShowInvert(&rDims);
+#endif
 			}
 		break;
 		}
@@ -937,8 +941,10 @@ Arrow::Track(POINT *p, anyOutput *o)
 void
 Label::ShowCursor(anyOutput *o)
 {
-	CalcRect(o);
-	ShowTextCursor(o, &Cursor, TextDef.ColTxt);
+	if(o) {
+		CalcRect(o);
+		if((o->OC_type & 0xff) == OC_BITMAP) ShowTextCursor(o, &Cursor, TextDef.ColTxt);
+		}
 }
 
 bool
@@ -1046,7 +1052,7 @@ TextFrame::ShowCursor(anyOutput *o)
 {
 	int cx, cy, w, h;
 
-	if(!o) return;
+	if(!o || (o->OC_type & 0xff) != OC_BITMAP) return;
 	TextDef.iSize = o->un2iy(TextDef.fSize);
 #ifdef _WINDOWS
 	linc = o->un2iy(TextDef.fSize*lspc);
@@ -1129,7 +1135,9 @@ Axis::DoMark(anyOutput *o, bool mark)
 		memcpy(&mrc, &rDims, sizeof(RECT));
 		IncrementMinMaxRect(&mrc, 6 + o->un2ix(axline.width));
 		mo = GetRectBitmap(&mrc, o);
-		InvertLine(pts, 2, &axline, &rDims, o, mark);
+		if(type & 0x04) InvertLine(gradient_box, 5, &axline, &rDims, o, mark);
+		else InvertLine(pts, 2, &axline, &rDims, o, mark);
+		o->UpdateRect(&mrc, false);
 		}
 	else RestoreRectBitmap(&mo, &mrc, o);
 }
@@ -1158,6 +1166,7 @@ Graph::ExecTool(MouseEvent *mev)
 			CurrDisp->HideMark();			CurrDisp->MouseCursor(MC_MOVE, false);
 			return true;
 		case MOUSE_MOVE:
+			defs.Idle(CMD_UPDATE);
 			if(TrackGO) {
 				if(TrackGO->Id == GO_DRAGHANDLE && TrackGO->type >= DH_18 &&
 					TrackGO->type <= DH_88) {
@@ -1285,6 +1294,7 @@ Graph::ExecTool(MouseEvent *mev)
 				((Graph*)PasteObj)->GRect.Ymax -= ((Graph*)PasteObj)->GRect.Ymin;
 				((Graph*)PasteObj)->GRect.Ymin = ((Graph*)PasteObj)->GRect.Xmin = 0.0;
 				PasteObj->Command(CMD_SCALE, &scale, 0L);
+				PasteObj->moveable = 1;
 				Command(CMD_DROP_GRAPH, (void*)PasteObj, 0L);
 				}
 			else {
